// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO32/DIO32_Linux_1.x_GSC_DN/driver/ioctl.c $
// $Rev: 54540 $
// $Date: 2024-05-28 09:45:15 -0500 (Tue, 28 May 2024) $

// DIO32: Device Driver: source file

#include "main.h"



// macros *********************************************************************

#define	INIT_START				D31			// BCR



//*****************************************************************************
static int _query(dev_data_t* dev, s32* arg)
{
	switch (arg[0])
	{
		default:							arg[0]	= DIO32_IOCTL_QUERY_ERROR;			break;
		case DIO32_QUERY_BOARD_INDEX:		arg[0]	= dev->board_index;					break;
		case DIO32_QUERY_BUS_WIDTH:			arg[0]	= dev->cache.bus_width;				break;
		case DIO32_QUERY_COUNT:				arg[0]	= DIO32_IOCTL_QUERY_LAST;			break;
		case DIO32_QUERY_DEVICE_TYPE:		arg[0]	= dev->board_type;					break;
		case DIO32_QUERY_FORM_FACTOR:		arg[0]	= dev->cache.form_factor;			break;
		case DIO32_QUERY_IO_BITS:			arg[0]	= dev->cache.port_qty;				break;
		case DIO32_QUERY_USER_JUMPER_QTY:	arg[0]	= dev->cache.user_jumper_qty;		break;
		case DIO32_QUERY_USER_JUMPER_SENSE:	arg[0]	= dev->cache.user_jumper_sense;		break;
		case DIO32_QUERY_USER_JUMPER_VALUE:	arg[0]	= dev->cache.user_jumper_value;		break;
		case DIO32_QUERY_XCVR_TYPE:			arg[0]	= dev->cache.xcvr_type;				break;
	}

	return(0);
}



//*****************************************************************************
int initialize_ioctl(dev_data_t* dev, void* arg)
{
	long		ms			= 5000;
	u32			reg;
	int			ret			= 0;
	int			tmp;
	VADDR_T		va			= dev->vaddr.gsc_bcr_32;

	// This device doesn't support an INITIALIZE DONE interrupt.

	// Initiate initialization.
	os_reg_mem_tx_u32(dev, va, INIT_START);

	// Manually wait for completion as this device dowsn't support an
	// Initialize Done interrupt.
	tmp	= gsc_poll_u32(dev, ms, va, INIT_START, 0);

	if (tmp)
	{
		ret	= ret ? ret : tmp;
		printf(	"%s: INITIALIZATION DID NOT COMPLETE WITHIN %ld ms.\n",
				dev->model,
				ms);
	}

	// Manually check for completion.
	reg	= os_reg_mem_rx_u32(dev, va);

	if (reg & INIT_START)
	{
		ret	= ret ? ret : -ETIMEDOUT;
		printf(	"%s: INITILIZE STILL ACTIVE AFTER %ld ms.\n",
				dev->model,
				ms);
	}


	return(ret);
}



//*****************************************************************************
static int _port_t_config(dev_data_t* dev, dio32_port_t* arg, VADDR_T va)
{
	u32	mask	= 0;
	int	ret		= 0;

	if (arg->port == -1)
		mask	= arg->mask;
	else if ((arg->port < 0) || (arg->port > 31))
		ret	= -EINVAL;
	else
		mask	= 1 << arg->port;

	switch (arg->action)
	{
		default:

			ret	= -EINVAL;
			break;

		case DIO32_ACT_RX:

			if (mask == 0)
				break;

			arg->value	= os_reg_mem_rx_u32(dev, va);
			arg->value	&= mask;
			break;

		case DIO32_ACT_MX:
		case DIO32_ACT_TX:

			if (mask == 0)
				break;

			if (mask == 0xFFFFFFFF)
			{
				os_reg_mem_tx_u32(dev, va, arg->value);
				break;
			}

			os_reg_mem_mx_u32(dev, va, arg->value, mask);
			break;
	}

	return(ret);
}



//*****************************************************************************
static int _response_config(dev_data_t* dev, dio32_port_t* arg, VADDR_T* va_list)
{
	u32		mask	= 0;
	int		ret		= 0;
	VADDR_T	va		= (VADDR_T) NULL;

	if ((arg->port < 0) || (arg->port > 31))
	{
		ret	= -EINVAL;
	}
	else
	{
		mask	= arg->mask;
		va		= va_list[arg->port];
	}

	switch (arg->action)
	{
		default:

			ret	= -EINVAL;
			break;

		case DIO32_ACT_RX:

			if (mask == 0)
				break;

			arg->value	= os_reg_mem_rx_u32(dev, va);
			arg->value	&= mask;
			break;

		case DIO32_ACT_MX:
		case DIO32_ACT_TX:

			if (mask == 0)
				break;

			if (mask == 0xFFFFFFFF)
			{
				os_reg_mem_tx_u32(dev, va, arg->value);
				break;
			}

			os_reg_mem_mx_u32(dev, va, arg->value, mask);
			break;
	}

	return(ret);
}



//*****************************************************************************
static int _deglitch_filter(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, 0xFF, dev->vaddr.gsc_dgcr_32, 7, 0);
	return(ret);
}



//*****************************************************************************
static int _deglitch_rate(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, 0xFFFFFF, dev->vaddr.gsc_dgcr_32, 31, 8);
	return(ret);
}



//*****************************************************************************
static int _hl_out_mask(dev_data_t* dev, dio32_port_t* arg)
{
	int	ret;

	ret	= _response_config(dev, arg, dev->vaddr.gsc_hl_out_msk);
	return(ret);
}



//*****************************************************************************
static int _hl_out_tri_state(dev_data_t* dev, dio32_port_t* arg)
{
	int	ret;
	ret	= _response_config(dev, arg, dev->vaddr.gsc_hl_out_tst);
	return(ret);
}



//*****************************************************************************
static int _hl_out_value(dev_data_t* dev, dio32_port_t* arg)
{
	int	ret;

	ret	= _response_config(dev, arg, dev->vaddr.gsc_hl_out_val);
	return(ret);
}



//*****************************************************************************
static int _io_in_deglitched_get(dev_data_t* dev, u32* arg)
{
	arg[0]	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_dgir_32);
	return(0);
}



//*****************************************************************************
static int _io_in_get(dev_data_t* dev, u32* arg)
{
	arg[0]	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_pir_32);
	return(0);
}



//*****************************************************************************
static int _io_out_clock(dev_data_t* dev, dio32_port_t* arg)
{
	static const s32	options[]	=
	{
		DIO32_IO_OUT_CLOCK_NONE,
		DIO32_IO_OUT_CLOCK_1,
		DIO32_IO_OUT_CLOCK_2,
		DIO32_IO_OUT_CLOCK_3,
		-1	// terminate list
	};

	u32	bit;
	int	end;
	u32	mask;
	int	ret		= 0;
	s32	val		= (s32) arg->value;

	switch (arg->port)
	{
		default:	mask	= 0xE;			break;	// invalid
		case -1:	mask	= arg->mask;	break;
		case 0:		mask	= 0x00000001;	break;
		case 4:		mask	= 0x00000010;	break;
		case 8:		mask	= 0x00000100;	break;
		case 12:	mask	= 0x00001000;	break;
		case 16:	mask	= 0x00010000;	break;
		case 20:	mask	= 0x00100000;	break;
		case 24:	mask	= 0x01000000;	break;
		case 28:	mask	= 0x10000000;	break;
	}

	switch (arg->action)
	{
		default:			mask		= 0xE;	break;	// invalid
		case DIO32_ACT_MX:	mask		= 0xE;	break;	// not supported
		case DIO32_ACT_RX:						break;
		case DIO32_ACT_TX:						break;
	}

	if (mask & 0xEEEEEEEE)
	{
		ret		= -EINVAL;
		mask	= 0;
	}

	for (end = 0; (ret == 0) && (end <= 31); end += 4)
	{
		bit	= 0x1 << end;

		if ((mask & bit) == 0)
			continue;

		switch (arg->action)
		{
			default:
			case DIO32_ACT_MX:

				ret	= -EINVAL;
				break;

			case DIO32_ACT_RX:

				val	= -1;

				FALLTHROUGH;	/* fall through */

			case DIO32_ACT_TX:

				ret			= gsc_s32_list_reg(
								dev,
								&val,
								options,
								dev->vaddr.gsc_ocr_32,
								end + 1,
								end);
				arg->value	= (u32) val;
				break;
		}
	}

	return(ret);
}



//*****************************************************************************
static int _io_out_dir(dev_data_t* dev, dio32_port_t* arg)
{
	int	ret;

	ret	= _port_t_config(dev, arg, dev->vaddr.gsc_iodr_32);
	return(ret);
}



//*****************************************************************************
static int _io_out_tri_state(dev_data_t* dev, dio32_port_t* arg)
{
	int	ret;

	ret	= _port_t_config(dev, arg, dev->vaddr.gsc_otsr_32);
	return(ret);
}



//*****************************************************************************
static int _io_out_val(dev_data_t* dev, dio32_port_t* arg)
{
	int	ret;

	ret	= _port_t_config(dev, arg, dev->vaddr.gsc_ovr_32);
	return(ret);
}



//*****************************************************************************
static int _irq_hl_enable(dev_data_t* dev, dio32_port_t* arg)
{
	int	ret;

	ret	= _port_t_config(dev, arg, dev->vaddr.gsc_hlier_32);
	return(ret);
}



//*****************************************************************************
static int _irq_lh_enable(dev_data_t* dev, dio32_port_t* arg)
{
	int	ret;

	ret	= _port_t_config(dev, arg, dev->vaddr.gsc_lhier_32);
	return(ret);
}



//*****************************************************************************
static int _ledx(dev_data_t* dev, s32* arg, int index)
{
	static const s32	options[]	=
	{
		DIO32_LED_OFF,
		DIO32_LED_GREEN,
		DIO32_LED_RED,
		DIO32_LED_BOTH,
		-1	// terminate list
	};

	int	end		= 2 * index;
	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, end + 1, end);
	return(ret);
}



//*****************************************************************************
static int _led0(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= _ledx(dev, arg, 0);
	return(ret);
}



//*****************************************************************************
static int _led1(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= _ledx(dev, arg, 1);
	return(ret);
}



//*****************************************************************************
static int _led2(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= _ledx(dev, arg, 2);
	return(ret);
}



//*****************************************************************************
static int _led3(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= _ledx(dev, arg, 3);
	return(ret);
}



//*****************************************************************************
static int _led4(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= _ledx(dev, arg, 4);
	return(ret);
}



//*****************************************************************************
static int _led5(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= _ledx(dev, arg, 5);
	return(ret);
}



//*****************************************************************************
static int _lh_out_mask(dev_data_t* dev, dio32_port_t* arg)
{
	int	ret;

	ret	= _response_config(dev, arg, dev->vaddr.gsc_lh_out_msk);
	return(ret);
}



//*****************************************************************************
static int _lh_out_tri_state(dev_data_t* dev, dio32_port_t* arg)
{
	int	ret;

	ret	= _response_config(dev, arg, dev->vaddr.gsc_lh_out_tst);
	return(ret);
}



//*****************************************************************************
static int _lh_out_value(dev_data_t* dev, dio32_port_t* arg)
{
	int	ret;

	ret	= _response_config(dev, arg, dev->vaddr.gsc_lh_out_val);
	return(ret);
}



// variables ******************************************************************

const gsc_ioctl_t	dev_ioctl_list[]	=
{
	{ DIO32_IOCTL_REG_READ,				(void*) gsc_reg_read_ioctl		},
	{ DIO32_IOCTL_REG_WRITE,			(void*) gsc_reg_write_ioctl		},
	{ DIO32_IOCTL_REG_MOD,				(void*) gsc_reg_mod_ioctl		},
	{ DIO32_IOCTL_QUERY,				(void*) _query					},
	{ DIO32_IOCTL_INITIALIZE,			(void*) initialize_ioctl		},
	{ DIO32_IOCTL_DEGLITCH_FILTER,		(void*) _deglitch_filter		},
	{ DIO32_IOCTL_DEGLITCH_RATE,		(void*) _deglitch_rate			},
	{ DIO32_IOCTL_HL_OUT_MASK,			(void*) _hl_out_mask			},
	{ DIO32_IOCTL_HL_OUT_TRI_STATE,		(void*) _hl_out_tri_state		},
	{ DIO32_IOCTL_HL_OUT_VALUE,			(void*) _hl_out_value			},
	{ DIO32_IOCTL_IO_IN_DEGLITCHED_GET,	(void*) _io_in_deglitched_get	},
	{ DIO32_IOCTL_IO_IN_GET,			(void*) _io_in_get				},
	{ DIO32_IOCTL_IO_OUT_CLOCK,			(void*) _io_out_clock			},
	{ DIO32_IOCTL_IO_OUT_DIR,			(void*) _io_out_dir				},
	{ DIO32_IOCTL_IO_OUT_TRI_STATE,		(void*) _io_out_tri_state		},
	{ DIO32_IOCTL_IO_OUT_VAL,			(void*) _io_out_val				},
	{ DIO32_IOCTL_IRQ_HL_ENABLE,		(void*) _irq_hl_enable			},
	{ DIO32_IOCTL_IRQ_LH_ENABLE,		(void*) _irq_lh_enable			},
	{ DIO32_IOCTL_LED0,					(void*) _led0					},
	{ DIO32_IOCTL_LED1,					(void*) _led1					},
	{ DIO32_IOCTL_LED2,					(void*) _led2					},
	{ DIO32_IOCTL_LED3,					(void*) _led3					},
	{ DIO32_IOCTL_LED4,					(void*) _led4					},
	{ DIO32_IOCTL_LED5,					(void*) _led5					},
	{ DIO32_IOCTL_LH_OUT_MASK,			(void*) _lh_out_mask			},
	{ DIO32_IOCTL_LH_OUT_TRI_STATE,		(void*) _lh_out_tri_state		},
	{ DIO32_IOCTL_LH_OUT_VALUE,			(void*) _lh_out_value			},
	{ DIO32_IOCTL_WAIT_EVENT,			(void*) gsc_wait_event_ioctl	},
	{ DIO32_IOCTL_WAIT_CANCEL,			(void*) gsc_wait_cancel_ioctl	},
	{ DIO32_IOCTL_WAIT_STATUS,			(void*) gsc_wait_status_ioctl	},

	{ -1, NULL }
};


