// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO32/DIO32_Linux_1.x_GSC_DN/driver/irq.c $
// $Rev: 45659 $
// $Date: 2019-08-05 17:46:43 -0500 (Mon, 05 Aug 2019) $

// DIO32: Device Driver: source file

#include "main.h"



//*****************************************************************************
void dev_irq_isr_local_handler(dev_data_t* dev)
{
	u32	count	= 0;
	int	disable	= 0;
	u32	isr;
	u32	itr;

	for (;;)
	{
		itr	= os_reg_mem_rx_u32(NULL, dev->vaddr.gsc_itr_32);
		itr	&= D0 | D1;

		if (itr == 0)
		{
			if (count == 0)
			{
				// We don't know the source of the interrupt.
				gsc_wait_resume_irq_main(dev, GSC_WAIT_MAIN_SPURIOUS);
			}

			break;
		}

		if (itr & D0)
		{
			// There are one or more Low-to-High interrupts.
			isr	= os_reg_mem_rx_u32(NULL, dev->vaddr.gsc_lhiscr_32);

			if (isr)
			{
				// Clear the interrupt.
				os_reg_mem_tx_u32(NULL, dev->vaddr.gsc_lhiscr_32, isr);

				// Resume any threads awiting any of these interrupts.
				gsc_wait_resume_irq_gsc(dev, DIO32_WAIT_GSC_LOW_HI);
				gsc_wait_resume_irq_io(dev, isr);
			}

			if (disable)
				os_reg_mem_mx_u32(NULL, dev->vaddr.gsc_lhier_32, 0, isr);
		}

		if (itr & D1)
		{
			// There are one or more High-to-Low interrupts.
			isr	= os_reg_mem_rx_u32(NULL, dev->vaddr.gsc_hliscr_32);

			if (isr)
			{
				// Clear the interrupt.
				os_reg_mem_tx_u32(NULL, dev->vaddr.gsc_hliscr_32, isr);

				// Resume any threads awiting any of these interrupts.
				gsc_wait_resume_irq_gsc(dev, DIO32_WAIT_GSC_HI_LOW);
				gsc_wait_resume_irq_alt(dev, isr);
			}

			if (disable)
				os_reg_mem_mx_u32(NULL, dev->vaddr.gsc_hlier_32, 0, isr);
		}

		if (disable)
			break;

		count++;

		if (count >= 100)
			disable	= 1;
	}
}



//*****************************************************************************
int dev_irq_create(dev_data_t* dev)
{
	int	ret;

	os_reg_mem_tx_u32(dev, dev->vaddr.gsc_hlier_32, 0);				// Disable IRQs.
	os_reg_mem_tx_u32(dev, dev->vaddr.gsc_lhier_32, 0);				// Disable IRQs.
	os_reg_mem_tx_u32(dev, dev->vaddr.gsc_hliscr_32, 0xFFFFFFFF);	// Clear IRQs.
	os_reg_mem_tx_u32(dev, dev->vaddr.gsc_lhiscr_32, 0xFFFFFFFF);	// Clear IRQs.
	ret	= gsc_irq_create(dev);
	return(ret);
}



//*****************************************************************************
void dev_irq_destroy(dev_data_t* dev)
{
	if (dev->vaddr.gsc_hlier_32)
		os_reg_mem_tx_u32(dev, dev->vaddr.gsc_hlier_32, 0);				// Disable IRQs.

	if (dev->vaddr.gsc_lhier_32)
		os_reg_mem_tx_u32(dev, dev->vaddr.gsc_lhier_32, 0);				// Disable IRQs.

	if (dev->vaddr.gsc_hliscr_32)
		os_reg_mem_tx_u32(dev, dev->vaddr.gsc_hliscr_32, 0xFFFFFFFF);	// Clear IRQs.

	if (dev->vaddr.gsc_lhiscr_32)
		os_reg_mem_tx_u32(dev, dev->vaddr.gsc_lhiscr_32, 0xFFFFFFFF);	// Clear IRQs.

	gsc_irq_destroy(dev);
}


