// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO32/DIO32_Linux_1.x_GSC_DN/driver/main.h $
// $Rev: 54540 $
// $Date: 2024-05-28 09:45:15 -0500 (Tue, 28 May 2024) $

// DIO32: Device Driver: header file

#ifndef	__MAIN_H__
#define	__MAIN_H__

#include "dio32.h"

#define	DEV_BAR_SHOW						0
#define	DEV_METRICS_SHOW					0
#define	DEV_PCI_ID_SHOW						0
#define	DEV_SUPPORTS_IRQ
#define	DEV_SUPPORTS_PROC_ID_STR
#define	DEV_SUPPORTS_WAIT
#define	GSC_ALT_DEV_GET(a)					(a)
#define	GSC_ALT_STRUCT_GET(i,d)				(d)
#define	GSC_ALT_STRUCT_T					dev_data_t
#define	GSC_DEVS_PER_BOARD					1
typedef struct _dev_data_t					dev_data_t;

#include "gsc_main.h"



// macros *********************************************************************

#define	DEV_MODEL							"DIO32"		// Upper case form of the below.
#define	DEV_NAME							"dio32"		// MUST AGREE WITH DIO32_BASE_NAME
#define	DEV_NAME_LC							DEV_NAME
#define	DEV_NAME_UC							DEV_MODEL

#define	DEV_VERSION_STR						GSC_STR(DEV_VERSION_MAJ_NUM) "." GSC_STR(DEV_VERSION_MIN_NUM)
#define	DEV_VERSION							DEV_VERSION_STR

#define	DEV_VERSION_MAJ_NUM					1
#define	DEV_VERSION_MIN_NUM					6
// 1.6	Updated to support the 6.x series kernel.
//		Updated for Fedora 38.
//		Updated comments for consistency.
//		Reorganized and updated ioctl.c for consistency.
//		Updated the make file for consistency.
//		Updated the release script for condistency.
// 1.5	Removed DMA code as it is unused.
//		Removed compiler warnings for Fedora 31.
//		Updated for porting to Windows.
//		Standardized various IOCTL system log messages.
//		Standardized processing of the initialization IOCTL service.
// 1.4	Added support for the 5.x kernel series.
//		Added support for Big Endian hosts.
//		Modified code so open succeeds even if initialization fails.
// 1.3	Bug fix: was ignoring BAR setup return value.
//		Updated per changes to the OS specific PCI services.
//		Updated to use the newer common driver sources.
//		Correct access to BAR0 and BAR1 (is RO, was RW).
//		Corrected the IOCTL code for the register write and mod services.
//		Made updates for DEV_SUPPORTS_XXX macro changes.
// 1.2	Modified the version string ao support Windows.
//		The BAR code has been updated to include the data under a single structure.
//		The register definitions have been updated.
//		Modified to allow multiple apps to simultaneously access a single device.
//		Removed some dead code.
//		LINTed the sources.
// 1.1	Made PCI registers accessible from inside an ISR.
//		White space cleanup.
//		Minor initialization improvements.
//		Updates for Jungo support.
// 1.0	Initial release.

// WAIT services
#define	DEV_WAIT_GSC_ALL					DIO32_WAIT_GSC_ALL
#define	DEV_WAIT_ALT_ALL					DIO32_WAIT_HI_LOW_ALL
#define	DEV_WAIT_IO_ALL						DIO32_WAIT_LOW_HI_ALL



// data types *****************************************************************

// This is not used, but is required.
struct _dev_io_t
{
	// Initialized by open and IOCTL services (initialize and/or service specific).


	// Initialized by I/O service at time of use.

	gsc_dma_ch_t*		dma_channel;		// Not used, but required.

	// Initialized by device specific I/O create code.

	u32					io_reg_offset;		// Offset of device's I/O FIFO.

	// Initialize by GSC commone create code.

};

struct _dev_data_t
{
	// Put an integral data type at beginning of structure.
	int					board_index;	// Index of the device being accessed.
	gsc_dev_type_t		board_type;		// Corresponds to basic device type.
	const char*			model;			// Base model number in upper case.
	int					share;			// Were we opened in shared mode?
	int					users;			// Number of currently active open() requests.
	char				proc_id_str[8];	// User Jumper setting, if supported.

	os_pci_t			pci;			// The kernel PCI device descriptor.
	os_data_t			os;				// OS specific data.
	os_spinlock_t		spinlock;		// Control ISR access.
	os_sem_t			sem;			// Control thread access.

	gsc_bar_t			bar;			// device register mappings
	gsc_dma_t			dma;			// For interrupts only.
	gsc_irq_t			irq;			// For interrupts.
	gsc_wait_node_t*	wait_list;

	struct
	{
		// The DMA registers are not used, but required.
		VADDR_T			plx_intcsr_32;		// Interrupt Control/Status Register
		VADDR_T			plx_dmaarb_32;		// DMA Arbitration Register
		VADDR_T			plx_dmathr_32;		// DMA Threshold Register

		VADDR_T			gsc_bcr_32;			// Board Control Register
		VADDR_T			gsc_bsr_32;			// Board Status Register

		VADDR_T			gsc_dgcr_32;		// Deglitch Control Register
		VADDR_T			gsc_dgir_32;		// Deglitch Input Register

		VADDR_T			gsc_fr_32;			// Features Register
		VADDR_T			gsc_frr_32;			// Firmware Revision Register

		VADDR_T			gsc_hl_out_msk[32];	// Hi-Low Mask
		VADDR_T			gsc_hl_out_tst[32];	// Hi-Low Tri-State
		VADDR_T			gsc_hl_out_val[32];	// Hi-Low Value
		VADDR_T			gsc_hlier_32;		// Hi-Low Interrupt Enable Register
		VADDR_T			gsc_hliscr_32;		// Hi-Low Interrupt Status/Clear Register

		VADDR_T			gsc_iodr_32;		// I/O Direction Register
		VADDR_T			gsc_itr_32;			// Interrupt Type Register

		VADDR_T			gsc_lh_out_msk[32];	// Low-Hi Mask
		VADDR_T			gsc_lh_out_tst[32];	// Low-Hi Tri-State
		VADDR_T			gsc_lh_out_val[32];	// Low-Hi Value
		VADDR_T			gsc_lhier_32;		// Low-Hi Interrupt Enable Register
		VADDR_T			gsc_lhiscr_32;		// Low-Hi Interrupt Status/Clear Register

		VADDR_T			gsc_ocr_32;			// Output Clock Register
		VADDR_T			gsc_otsr_32;		// Output Tri-State Register
		VADDR_T			gsc_ovr_32;			// Output Value Register

		VADDR_T			gsc_pcifr_32;		// Programmable Clock 1 Register
		VADDR_T			gsc_pc1pwmdr_32;	// Programmable Clock 1 Register
		VADDR_T			gsc_pc1r_32;		// Programmable Clock 1 Register
		VADDR_T			gsc_pc2fr_32;		// Programmable Clock 1 Register
		VADDR_T			gsc_pc2pwmdr_32;	// Programmable Clock 1 Register
		VADDR_T			gsc_pc2r_32;		// Programmable Clock 1 Register
		VADDR_T			gsc_pc3fr_32;		// Programmable Clock 1 Register
		VADDR_T			gsc_pc3pwmdr_32;	// Programmable Clock 1 Register
		VADDR_T			gsc_pc3r_32;		// Programmable Clock 1 Register
		VADDR_T			gsc_pir_32;			// Pin Input Register

	} vaddr;

	struct
	{
		s32				bus_width;			// 64-bit or 32-bit

		s32				form_factor;		// PCI, PMC, ...

		u32				gsc_fr_32;			// Features Register
		u32				gsc_frr_32;			// Firmware Revision Register

		s32				port_qty;			// Number of I/O port pins.

		s32				user_jumper_qty;	// How many user jumpers are supported?
		s32				user_jumper_sense;	// What bit value means "jumper installed"?
		s32				user_jumper_value;	// The value read from the device.

		s32				xcvr_type;			// If known.

	} cache;
};



// prototypes *****************************************************************

int		dev_irq_create(dev_data_t* dev);
void	dev_irq_destroy(dev_data_t* dev);

int		initialize_ioctl(dev_data_t* dev, void* arg);



#endif
