// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO32/DIO32_Linux_1.x_GSC_DN/driver/dio32.h $
// $Rev: 51811 $
// $Date: 2022-10-14 12:50:52 -0500 (Fri, 14 Oct 2022) $

// DIO32: Device Driver: header file

#ifndef	__DIO32_H__
#define	__DIO32_H__

#include "gsc_common.h"
#include "gsc_pci9056.h"



// macros *********************************************************************

#define	DIO32_BASE_NAME						"dio32"
#define	DIO32_BASE_NAME_LC					DIO32_BASE_NAME
#define	DIO32_BASE_NAME_UC					"DIO32"

// IOCTL command codes
#define	DIO32_IOCTL_REG_READ				OS_IOCTL_RW( 0, 12, gsc_reg_t)
#define	DIO32_IOCTL_REG_WRITE				OS_IOCTL_W ( 1, 12, gsc_reg_t)
#define	DIO32_IOCTL_REG_MOD					OS_IOCTL_W ( 2, 12, gsc_reg_t)
#define	DIO32_IOCTL_QUERY					OS_IOCTL_RW( 3,  4, s32)
#define	DIO32_IOCTL_INITIALIZE				OS_IOCTL   ( 4)
#define	DIO32_IOCTL_DEGLITCH_FILTER			OS_IOCTL_RW( 5,  4, s32)
#define	DIO32_IOCTL_DEGLITCH_RATE			OS_IOCTL_RW( 6,  4, s32)
#define	DIO32_IOCTL_HL_OUT_MASK				OS_IOCTL_RW( 7, 16, dio32_port_t)
#define	DIO32_IOCTL_HL_OUT_TRI_STATE		OS_IOCTL_RW( 8, 16, dio32_port_t)
#define	DIO32_IOCTL_HL_OUT_VALUE			OS_IOCTL_RW( 9, 16, dio32_port_t)
#define	DIO32_IOCTL_IO_IN_DEGLITCHED_GET	OS_IOCTL_R (10,  4, u32)
#define	DIO32_IOCTL_IO_IN_GET				OS_IOCTL_R (11,  4, u32)
#define	DIO32_IOCTL_IO_OUT_CLOCK			OS_IOCTL_RW(12, 16, dio32_port_t)
#define	DIO32_IOCTL_IO_OUT_DIR				OS_IOCTL_RW(13, 16, dio32_port_t)
#define	DIO32_IOCTL_IO_OUT_TRI_STATE		OS_IOCTL_RW(14, 16, dio32_port_t)
#define	DIO32_IOCTL_IO_OUT_VAL				OS_IOCTL_RW(15, 16, dio32_port_t)
#define	DIO32_IOCTL_IRQ_HL_ENABLE			OS_IOCTL_RW(16, 16, dio32_port_t)
#define	DIO32_IOCTL_IRQ_LH_ENABLE			OS_IOCTL_RW(17, 16, dio32_port_t)
#define	DIO32_IOCTL_LED0					OS_IOCTL_RW(18,  4, s32)
#define	DIO32_IOCTL_LED1					OS_IOCTL_RW(19,  4, s32)
#define	DIO32_IOCTL_LED2					OS_IOCTL_RW(20,  4, s32)
#define	DIO32_IOCTL_LED3					OS_IOCTL_RW(21,  4, s32)
#define	DIO32_IOCTL_LED4					OS_IOCTL_RW(22,  4, s32)
#define	DIO32_IOCTL_LED5					OS_IOCTL_RW(23,  4, s32)
#define	DIO32_IOCTL_LH_OUT_MASK				OS_IOCTL_RW(24, 16, dio32_port_t)
#define	DIO32_IOCTL_LH_OUT_TRI_STATE		OS_IOCTL_RW(25, 16, dio32_port_t)
#define	DIO32_IOCTL_LH_OUT_VALUE			OS_IOCTL_RW(26, 16, dio32_port_t)
#define	DIO32_IOCTL_WAIT_EVENT				OS_IOCTL_RW(27, 28, dio32_wait_t)
#define	DIO32_IOCTL_WAIT_CANCEL				OS_IOCTL_RW(28, 28, dio32_wait_t)
#define	DIO32_IOCTL_WAIT_STATUS				OS_IOCTL_RW(29, 28, dio32_wait_t)

#define	DIO32_ACT_MX						0	// perform a read-modify-write
#define	DIO32_ACT_RX						1	// read the current settings
#define	DIO32_ACT_TX						2	// overwrite the current settings



// data types *****************************************************************

typedef struct
{
	s32	port;		// -1 for all supported, or index of desired port
	s32	action;		// DIO32_ACT_MX/RX/TX
	u32	value;		// The per port value, usually bit-per-port.
	u32	mask;		// Mask of ports to access for DIO32_ACT_MX, else zero.
} dio32_port_t;

typedef struct
{
	u32	flags;		// done, timeout, cancel, ...
	u32	main;		// interrupts: PCI, DMA0, DMA1, Local, Other
	u32	gsc;		// Various interrupt options.
	u32	hi_low;		// Individual High-to-Low interrupt sources.
	u32	low_hi;		// Individual Low-to-High interrupt sources.
	u32	timeout_ms;	// milliseconds
	u32	count;		// status: number of awaiting threads meeting any criteria
					// cancel: number of waits cancelled
} dio32_wait_t;



//*****************************************************************************
// DIO32_IOCTL_REG_READ
// DIO32_IOCTL_REG_WRITE
// DIO32_IOCTL_REG_MOD
//
#define	DIO32_REG_ENCODE(s,o)				GSC_REG_ENCODE(GSC_REG_TYPE_BAR2,(s),(o))
// Parameter:	gsc_reg_t*
#define	DIO32_GSC_FRR						DIO32_REG_ENCODE(4,0x000)// Firmware Revision Register
#define	DIO32_GSC_FR						DIO32_REG_ENCODE(4,0x004)// Features Register
#define	DIO32_GSC_BCR						DIO32_REG_ENCODE(4,0x008)// Board Control Register
#define	DIO32_GSC_BSR						DIO32_REG_ENCODE(4,0x00C)// Board Status Register
#define	DIO32_GSC_ITR						DIO32_REG_ENCODE(4,0x010)// Interrupt Type Register

#define	DIO32_GSC_IODR						DIO32_REG_ENCODE(4,0x040)// I/O Direction Register
#define	DIO32_GSC_PIR						DIO32_REG_ENCODE(4,0x044)// Pin Input Register
#define	DIO32_GSC_DGIR						DIO32_REG_ENCODE(4,0x048)// Deglitch Input Register

#define	DIO32_GSC_OTSR						DIO32_REG_ENCODE(4,0x050)// Output Tri-State Register
#define	DIO32_GSC_OVR						DIO32_REG_ENCODE(4,0x054)// Output Value Register
#define	DIO32_GSC_OCR						DIO32_REG_ENCODE(4,0x058)// Output Clock Register

#define	DIO32_GSC_DGCR						DIO32_REG_ENCODE(4,0x070)// Deglitch Control Register

#define	DIO32_GSC_PC1R						DIO32_REG_ENCODE(4,0x080)// Programmable Clock 1 Register
#define	DIO32_GSC_PC1PWMDR					DIO32_REG_ENCODE(4,0x084)// Programmable Clock 1 Register
#define	DIO32_GSC_PC1FR						DIO32_REG_ENCODE(4,0x088)// Programmable Clock 1 Register

#define	DIO32_GSC_PC2R						DIO32_REG_ENCODE(4,0x090)// Programmable Clock 2 Register
#define	DIO32_GSC_PC2PWMDR					DIO32_REG_ENCODE(4,0x094)// Programmable Clock 1 Register
#define	DIO32_GSC_PC2FR						DIO32_REG_ENCODE(4,0x098)// Programmable Clock 1 Register

#define	DIO32_GSC_PC3R						DIO32_REG_ENCODE(4,0x0A0)// Programmable Clock 3 Register
#define	DIO32_GSC_PC3PWMDR					DIO32_REG_ENCODE(4,0x0A4)// Programmable Clock 1 Register
#define	DIO32_GSC_PC3FR						DIO32_REG_ENCODE(4,0x0A8)// Programmable Clock 1 Register

#define	DIO32_GSC_LHIER						DIO32_REG_ENCODE(4,0x180)// Low-Hi Interrupt Enable Register
#define	DIO32_GSC_LHISCR					DIO32_REG_ENCODE(4,0x184)// Low-Hi Interrupt Status/Clear Register
#define	DIO32_GSC_HLIER						DIO32_REG_ENCODE(4,0x188)// Hi-Low Interrupt Enable Register
#define	DIO32_GSC_HLISCR					DIO32_REG_ENCODE(4,0x18C)// Hi-Low Interrupt Status/Clear Register

#define	DIO32_GSC_D00LHOMR					DIO32_REG_ENCODE(4,0x200)// D0 Low-Hi Output Mask Register
#define	DIO32_GSC_D00LHOTSR					DIO32_REG_ENCODE(4,0x204)// D0 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D00LHOVR					DIO32_REG_ENCODE(4,0x208)// D0 Low-Hi Output Value Register

#define	DIO32_GSC_D00HLOMR					DIO32_REG_ENCODE(4,0x210)// D0 Low-Hi Output Mask Register
#define	DIO32_GSC_D00HLOTSR					DIO32_REG_ENCODE(4,0x214)// D0 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D00HLOVR					DIO32_REG_ENCODE(4,0x218)// D0 Low-Hi Output Value Register

#define	DIO32_GSC_D01LHOMR					DIO32_REG_ENCODE(4,0x220)// D1 Low-Hi Output Mask Register
#define	DIO32_GSC_D01LHOTSR					DIO32_REG_ENCODE(4,0x224)// D1 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D01LHOVR					DIO32_REG_ENCODE(4,0x228)// D1 Low-Hi Output Value Register

#define	DIO32_GSC_D01HLOMR					DIO32_REG_ENCODE(4,0x230)// D1 Low-Hi Output Mask Register
#define	DIO32_GSC_D01HLOTSR					DIO32_REG_ENCODE(4,0x234)// D1 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D01HLOVR					DIO32_REG_ENCODE(4,0x238)// D1 Low-Hi Output Value Register

#define	DIO32_GSC_D02LHOMR					DIO32_REG_ENCODE(4,0x240)// D2 Low-Hi Output Mask Register
#define	DIO32_GSC_D02LHOTSR					DIO32_REG_ENCODE(4,0x244)// D2 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D02LHOVR					DIO32_REG_ENCODE(4,0x248)// D2 Low-Hi Output Value Register

#define	DIO32_GSC_D02HLOMR					DIO32_REG_ENCODE(4,0x250)// D2 Low-Hi Output Mask Register
#define	DIO32_GSC_D02HLOTSR					DIO32_REG_ENCODE(4,0x254)// D2 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D02HLOVR					DIO32_REG_ENCODE(4,0x258)// D2 Low-Hi Output Value Register

#define	DIO32_GSC_D03LHOMR					DIO32_REG_ENCODE(4,0x260)// D3 Low-Hi Output Mask Register
#define	DIO32_GSC_D03LHOTSR					DIO32_REG_ENCODE(4,0x264)// D3 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D03LHOVR					DIO32_REG_ENCODE(4,0x268)// D3 Low-Hi Output Value Register

#define	DIO32_GSC_D03HLOMR					DIO32_REG_ENCODE(4,0x270)// D3 Low-Hi Output Mask Register
#define	DIO32_GSC_D03HLOTSR					DIO32_REG_ENCODE(4,0x274)// D3 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D03HLOVR					DIO32_REG_ENCODE(4,0x278)// D3 Low-Hi Output Value Register

#define	DIO32_GSC_D04LHOMR					DIO32_REG_ENCODE(4,0x280)// D4 Low-Hi Output Mask Register
#define	DIO32_GSC_D04LHOTSR					DIO32_REG_ENCODE(4,0x284)// D4 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D04LHOVR					DIO32_REG_ENCODE(4,0x288)// D4 Low-Hi Output Value Register

#define	DIO32_GSC_D04HLOMR					DIO32_REG_ENCODE(4,0x290)// D4 Low-Hi Output Mask Register
#define	DIO32_GSC_D04HLOTSR					DIO32_REG_ENCODE(4,0x294)// D4 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D04HLOVR					DIO32_REG_ENCODE(4,0x298)// D4 Low-Hi Output Value Register

#define	DIO32_GSC_D05LHOMR					DIO32_REG_ENCODE(4,0x2A0)// D5 Low-Hi Output Mask Register
#define	DIO32_GSC_D05LHOTSR					DIO32_REG_ENCODE(4,0x2A4)// D5 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D05LHOVR					DIO32_REG_ENCODE(4,0x2A8)// D5 Low-Hi Output Value Register

#define	DIO32_GSC_D05HLOMR					DIO32_REG_ENCODE(4,0x2B0)// D5 Low-Hi Output Mask Register
#define	DIO32_GSC_D05HLOTSR					DIO32_REG_ENCODE(4,0x2B4)// D5 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D05HLOVR					DIO32_REG_ENCODE(4,0x2B8)// D5 Low-Hi Output Value Register

#define	DIO32_GSC_D06LHOMR					DIO32_REG_ENCODE(4,0x2C0)// D6 Low-Hi Output Mask Register
#define	DIO32_GSC_D06LHOTSR					DIO32_REG_ENCODE(4,0x2C4)// D6 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D06LHOVR					DIO32_REG_ENCODE(4,0x2C8)// D6 Low-Hi Output Value Register

#define	DIO32_GSC_D06HLOMR					DIO32_REG_ENCODE(4,0x2D0)// D6 Low-Hi Output Mask Register
#define	DIO32_GSC_D06HLOTSR					DIO32_REG_ENCODE(4,0x2D4)// D6 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D06HLOVR					DIO32_REG_ENCODE(4,0x2D8)// D6 Low-Hi Output Value Register

#define	DIO32_GSC_D07LHOMR					DIO32_REG_ENCODE(4,0x2E0)// D7 Low-Hi Output Mask Register
#define	DIO32_GSC_D07LHOTSR					DIO32_REG_ENCODE(4,0x2E4)// D7 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D07LHOVR					DIO32_REG_ENCODE(4,0x2E8)// D7 Low-Hi Output Value Register

#define	DIO32_GSC_D07HLOMR					DIO32_REG_ENCODE(4,0x2F0)// D7 Low-Hi Output Mask Register
#define	DIO32_GSC_D07HLOTSR					DIO32_REG_ENCODE(4,0x2F4)// D7 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D07HLOVR					DIO32_REG_ENCODE(4,0x2F8)// D7 Low-Hi Output Value Register

#define	DIO32_GSC_D08LHOMR					DIO32_REG_ENCODE(4,0x300)// D8 Low-Hi Output Mask Register
#define	DIO32_GSC_D08LHOTSR					DIO32_REG_ENCODE(4,0x304)// D8 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D08LHOVR					DIO32_REG_ENCODE(4,0x308)// D8 Low-Hi Output Value Register

#define	DIO32_GSC_D08HLOMR					DIO32_REG_ENCODE(4,0x310)// D8 Low-Hi Output Mask Register
#define	DIO32_GSC_D08HLOTSR					DIO32_REG_ENCODE(4,0x314)// D8 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D08HLOVR					DIO32_REG_ENCODE(4,0x318)// D8 Low-Hi Output Value Register

#define	DIO32_GSC_D09LHOMR					DIO32_REG_ENCODE(4,0x320)// D9 Low-Hi Output Mask Register
#define	DIO32_GSC_D09LHOTSR					DIO32_REG_ENCODE(4,0x324)// D9 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D09LHOVR					DIO32_REG_ENCODE(4,0x328)// D9 Low-Hi Output Value Register

#define	DIO32_GSC_D09HLOMR					DIO32_REG_ENCODE(4,0x330)// D9 Low-Hi Output Mask Register
#define	DIO32_GSC_D09HLOTSR					DIO32_REG_ENCODE(4,0x334)// D9 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D09HLOVR					DIO32_REG_ENCODE(4,0x338)// D9 Low-Hi Output Value Register

#define	DIO32_GSC_D10LHOMR					DIO32_REG_ENCODE(4,0x340)// D10 Low-Hi Output Mask Register
#define	DIO32_GSC_D10LHOTSR					DIO32_REG_ENCODE(4,0x344)// D10 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D10LHOVR					DIO32_REG_ENCODE(4,0x348)// D10 Low-Hi Output Value Register

#define	DIO32_GSC_D10HLOMR					DIO32_REG_ENCODE(4,0x350)// D10 Low-Hi Output Mask Register
#define	DIO32_GSC_D10HLOTSR					DIO32_REG_ENCODE(4,0x354)// D10 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D10HLOVR					DIO32_REG_ENCODE(4,0x358)// D10 Low-Hi Output Value Register

#define	DIO32_GSC_D11LHOMR					DIO32_REG_ENCODE(4,0x360)// D11 Low-Hi Output Mask Register
#define	DIO32_GSC_D11LHOTSR					DIO32_REG_ENCODE(4,0x364)// D11 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D11LHOVR					DIO32_REG_ENCODE(4,0x368)// D11 Low-Hi Output Value Register

#define	DIO32_GSC_D11HLOMR					DIO32_REG_ENCODE(4,0x370)// D11 Low-Hi Output Mask Register
#define	DIO32_GSC_D11HLOTSR					DIO32_REG_ENCODE(4,0x374)// D11 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D11HLOVR					DIO32_REG_ENCODE(4,0x378)// D11 Low-Hi Output Value Register

#define	DIO32_GSC_D12LHOMR					DIO32_REG_ENCODE(4,0x380)// D12 Low-Hi Output Mask Register
#define	DIO32_GSC_D12LHOTSR					DIO32_REG_ENCODE(4,0x384)// D12 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D12LHOVR					DIO32_REG_ENCODE(4,0x388)// D12 Low-Hi Output Value Register

#define	DIO32_GSC_D12HLOMR					DIO32_REG_ENCODE(4,0x390)// D12 Low-Hi Output Mask Register
#define	DIO32_GSC_D12HLOTSR					DIO32_REG_ENCODE(4,0x394)// D12 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D12HLOVR					DIO32_REG_ENCODE(4,0x398)// D12 Low-Hi Output Value Register

#define	DIO32_GSC_D13LHOMR					DIO32_REG_ENCODE(4,0x3A0)// D13 Low-Hi Output Mask Register
#define	DIO32_GSC_D13LHOTSR					DIO32_REG_ENCODE(4,0x3A4)// D13 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D13LHOVR					DIO32_REG_ENCODE(4,0x3A8)// D13 Low-Hi Output Value Register

#define	DIO32_GSC_D13HLOMR					DIO32_REG_ENCODE(4,0x3B0)// D13 Low-Hi Output Mask Register
#define	DIO32_GSC_D13HLOTSR					DIO32_REG_ENCODE(4,0x3B4)// D13 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D13HLOVR					DIO32_REG_ENCODE(4,0x3B8)// D13 Low-Hi Output Value Register

#define	DIO32_GSC_D14LHOMR					DIO32_REG_ENCODE(4,0x3C0)// D14 Low-Hi Output Mask Register
#define	DIO32_GSC_D14LHOTSR					DIO32_REG_ENCODE(4,0x3C4)// D14 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D14LHOVR					DIO32_REG_ENCODE(4,0x3C8)// D14 Low-Hi Output Value Register

#define	DIO32_GSC_D14HLOMR					DIO32_REG_ENCODE(4,0x3D0)// D14 Low-Hi Output Mask Register
#define	DIO32_GSC_D14HLOTSR					DIO32_REG_ENCODE(4,0x3D4)// D14 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D14HLOVR					DIO32_REG_ENCODE(4,0x3D8)// D14 Low-Hi Output Value Register

#define	DIO32_GSC_D15LHOMR					DIO32_REG_ENCODE(4,0x3E0)// D15 Low-Hi Output Mask Register
#define	DIO32_GSC_D15LHOTSR					DIO32_REG_ENCODE(4,0x3E4)// D15 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D15LHOVR					DIO32_REG_ENCODE(4,0x3E8)// D15 Low-Hi Output Value Register

#define	DIO32_GSC_D15HLOMR					DIO32_REG_ENCODE(4,0x3F0)// D15 Low-Hi Output Mask Register
#define	DIO32_GSC_D15HLOTSR					DIO32_REG_ENCODE(4,0x3F4)// D15 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D15HLOVR					DIO32_REG_ENCODE(4,0x3F8)// D15 Low-Hi Output Value Register

#define	DIO32_GSC_D16LHOMR					DIO32_REG_ENCODE(4,0x400)// D16 Low-Hi Output Mask Register
#define	DIO32_GSC_D16LHOTSR					DIO32_REG_ENCODE(4,0x404)// D16 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D16LHOVR					DIO32_REG_ENCODE(4,0x408)// D16 Low-Hi Output Value Register

#define	DIO32_GSC_D16HLOMR					DIO32_REG_ENCODE(4,0x410)// D16 Low-Hi Output Mask Register
#define	DIO32_GSC_D16HLOTSR					DIO32_REG_ENCODE(4,0x414)// D16 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D16HLOVR					DIO32_REG_ENCODE(4,0x418)// D16 Low-Hi Output Value Register

#define	DIO32_GSC_D17LHOMR					DIO32_REG_ENCODE(4,0x420)// D17 Low-Hi Output Mask Register
#define	DIO32_GSC_D17LHOTSR					DIO32_REG_ENCODE(4,0x424)// D17 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D17LHOVR					DIO32_REG_ENCODE(4,0x428)// D17 Low-Hi Output Value Register

#define	DIO32_GSC_D17HLOMR					DIO32_REG_ENCODE(4,0x430)// D17 Low-Hi Output Mask Register
#define	DIO32_GSC_D17HLOTSR					DIO32_REG_ENCODE(4,0x434)// D17 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D17HLOVR					DIO32_REG_ENCODE(4,0x438)// D17 Low-Hi Output Value Register

#define	DIO32_GSC_D18LHOMR					DIO32_REG_ENCODE(4,0x440)// D18 Low-Hi Output Mask Register
#define	DIO32_GSC_D18LHOTSR					DIO32_REG_ENCODE(4,0x444)// D18 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D18LHOVR					DIO32_REG_ENCODE(4,0x448)// D18 Low-Hi Output Value Register

#define	DIO32_GSC_D18HLOMR					DIO32_REG_ENCODE(4,0x450)// D18 Low-Hi Output Mask Register
#define	DIO32_GSC_D18HLOTSR					DIO32_REG_ENCODE(4,0x454)// D18 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D18HLOVR					DIO32_REG_ENCODE(4,0x458)// D18 Low-Hi Output Value Register

#define	DIO32_GSC_D19LHOMR					DIO32_REG_ENCODE(4,0x460)// D19 Low-Hi Output Mask Register
#define	DIO32_GSC_D19LHOTSR					DIO32_REG_ENCODE(4,0x464)// D19 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D19LHOVR					DIO32_REG_ENCODE(4,0x468)// D19 Low-Hi Output Value Register

#define	DIO32_GSC_D19HLOMR					DIO32_REG_ENCODE(4,0x470)// D19 Low-Hi Output Mask Register
#define	DIO32_GSC_D19HLOTSR					DIO32_REG_ENCODE(4,0x474)// D19 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D19HLOVR					DIO32_REG_ENCODE(4,0x478)// D19 Low-Hi Output Value Register

#define	DIO32_GSC_D20LHOMR					DIO32_REG_ENCODE(4,0x480)// D20 Low-Hi Output Mask Register
#define	DIO32_GSC_D20LHOTSR					DIO32_REG_ENCODE(4,0x484)// D20 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D20LHOVR					DIO32_REG_ENCODE(4,0x488)// D20 Low-Hi Output Value Register

#define	DIO32_GSC_D20HLOMR					DIO32_REG_ENCODE(4,0x490)// D20 Low-Hi Output Mask Register
#define	DIO32_GSC_D20HLOTSR					DIO32_REG_ENCODE(4,0x494)// D20 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D20HLOVR					DIO32_REG_ENCODE(4,0x498)// D20 Low-Hi Output Value Register

#define	DIO32_GSC_D21LHOMR					DIO32_REG_ENCODE(4,0x4A0)// D21 Low-Hi Output Mask Register
#define	DIO32_GSC_D21LHOTSR					DIO32_REG_ENCODE(4,0x4A4)// D21 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D21LHOVR					DIO32_REG_ENCODE(4,0x4A8)// D21 Low-Hi Output Value Register

#define	DIO32_GSC_D21HLOMR					DIO32_REG_ENCODE(4,0x4B0)// D21 Low-Hi Output Mask Register
#define	DIO32_GSC_D21HLOTSR					DIO32_REG_ENCODE(4,0x4B4)// D21 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D21HLOVR					DIO32_REG_ENCODE(4,0x4B8)// D21 Low-Hi Output Value Register

#define	DIO32_GSC_D22LHOMR					DIO32_REG_ENCODE(4,0x4C0)// D22 Low-Hi Output Mask Register
#define	DIO32_GSC_D22LHOTSR					DIO32_REG_ENCODE(4,0x4C4)// D22 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D22LHOVR					DIO32_REG_ENCODE(4,0x4C8)// D22 Low-Hi Output Value Register

#define	DIO32_GSC_D22HLOMR					DIO32_REG_ENCODE(4,0x4D0)// D22 Low-Hi Output Mask Register
#define	DIO32_GSC_D22HLOTSR					DIO32_REG_ENCODE(4,0x4D4)// D22 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D22HLOVR					DIO32_REG_ENCODE(4,0x4D8)// D22 Low-Hi Output Value Register

#define	DIO32_GSC_D23LHOMR					DIO32_REG_ENCODE(4,0x4E0)// D23 Low-Hi Output Mask Register
#define	DIO32_GSC_D23LHOTSR					DIO32_REG_ENCODE(4,0x4E4)// D23 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D23LHOVR					DIO32_REG_ENCODE(4,0x4E8)// D23 Low-Hi Output Value Register

#define	DIO32_GSC_D23HLOMR					DIO32_REG_ENCODE(4,0x4F0)// D23 Low-Hi Output Mask Register
#define	DIO32_GSC_D23HLOTSR					DIO32_REG_ENCODE(4,0x4F4)// D23 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D23HLOVR					DIO32_REG_ENCODE(4,0x4F8)// D23 Low-Hi Output Value Register

#define	DIO32_GSC_D24LHOMR					DIO32_REG_ENCODE(4,0x500)// D24 Low-Hi Output Mask Register
#define	DIO32_GSC_D24LHOTSR					DIO32_REG_ENCODE(4,0x504)// D24 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D24LHOVR					DIO32_REG_ENCODE(4,0x508)// D24 Low-Hi Output Value Register

#define	DIO32_GSC_D24HLOMR					DIO32_REG_ENCODE(4,0x510)// D24 Low-Hi Output Mask Register
#define	DIO32_GSC_D24HLOTSR					DIO32_REG_ENCODE(4,0x514)// D24 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D24HLOVR					DIO32_REG_ENCODE(4,0x518)// D24 Low-Hi Output Value Register

#define	DIO32_GSC_D25LHOMR					DIO32_REG_ENCODE(4,0x520)// D25 Low-Hi Output Mask Register
#define	DIO32_GSC_D25LHOTSR					DIO32_REG_ENCODE(4,0x524)// D25 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D25LHOVR					DIO32_REG_ENCODE(4,0x528)// D25 Low-Hi Output Value Register

#define	DIO32_GSC_D25HLOMR					DIO32_REG_ENCODE(4,0x530)// D25 Low-Hi Output Mask Register
#define	DIO32_GSC_D25HLOTSR					DIO32_REG_ENCODE(4,0x534)// D25 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D25HLOVR					DIO32_REG_ENCODE(4,0x538)// D25 Low-Hi Output Value Register

#define	DIO32_GSC_D26LHOMR					DIO32_REG_ENCODE(4,0x540)// D26 Low-Hi Output Mask Register
#define	DIO32_GSC_D26LHOTSR					DIO32_REG_ENCODE(4,0x544)// D26 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D26LHOVR					DIO32_REG_ENCODE(4,0x548)// D26 Low-Hi Output Value Register

#define	DIO32_GSC_D26HLOMR					DIO32_REG_ENCODE(4,0x550)// D26 Low-Hi Output Mask Register
#define	DIO32_GSC_D26HLOTSR					DIO32_REG_ENCODE(4,0x554)// D26 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D26HLOVR					DIO32_REG_ENCODE(4,0x558)// D26 Low-Hi Output Value Register

#define	DIO32_GSC_D27LHOMR					DIO32_REG_ENCODE(4,0x560)// D27 Low-Hi Output Mask Register
#define	DIO32_GSC_D27LHOTSR					DIO32_REG_ENCODE(4,0x564)// D27 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D27LHOVR					DIO32_REG_ENCODE(4,0x568)// D27 Low-Hi Output Value Register

#define	DIO32_GSC_D27HLOMR					DIO32_REG_ENCODE(4,0x570)// D27 Low-Hi Output Mask Register
#define	DIO32_GSC_D27HLOTSR					DIO32_REG_ENCODE(4,0x574)// D27 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D27HLOVR					DIO32_REG_ENCODE(4,0x578)// D27 Low-Hi Output Value Register

#define	DIO32_GSC_D28LHOMR					DIO32_REG_ENCODE(4,0x580)// D28 Low-Hi Output Mask Register
#define	DIO32_GSC_D28LHOTSR					DIO32_REG_ENCODE(4,0x584)// D28 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D28LHOVR					DIO32_REG_ENCODE(4,0x588)// D28 Low-Hi Output Value Register

#define	DIO32_GSC_D28HLOMR					DIO32_REG_ENCODE(4,0x590)// D28 Low-Hi Output Mask Register
#define	DIO32_GSC_D28HLOTSR					DIO32_REG_ENCODE(4,0x594)// D28 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D28HLOVR					DIO32_REG_ENCODE(4,0x598)// D28 Low-Hi Output Value Register

#define	DIO32_GSC_D29LHOMR					DIO32_REG_ENCODE(4,0x5A0)// D29 Low-Hi Output Mask Register
#define	DIO32_GSC_D29LHOTSR					DIO32_REG_ENCODE(4,0x5A4)// D29 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D29LHOVR					DIO32_REG_ENCODE(4,0x5A8)// D29 Low-Hi Output Value Register

#define	DIO32_GSC_D29HLOMR					DIO32_REG_ENCODE(4,0x5B0)// D29 Low-Hi Output Mask Register
#define	DIO32_GSC_D29HLOTSR					DIO32_REG_ENCODE(4,0x5B4)// D29 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D29HLOVR					DIO32_REG_ENCODE(4,0x5B8)// D29 Low-Hi Output Value Register

#define	DIO32_GSC_D30LHOMR					DIO32_REG_ENCODE(4,0x5C0)// D30 Low-Hi Output Mask Register
#define	DIO32_GSC_D30LHOTSR					DIO32_REG_ENCODE(4,0x5C4)// D30 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D30LHOVR					DIO32_REG_ENCODE(4,0x5C8)// D30 Low-Hi Output Value Register

#define	DIO32_GSC_D30HLOMR					DIO32_REG_ENCODE(4,0x5D0)// D30 Low-Hi Output Mask Register
#define	DIO32_GSC_D30HLOTSR					DIO32_REG_ENCODE(4,0x5D4)// D30 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D30HLOVR					DIO32_REG_ENCODE(4,0x5D8)// D30 Low-Hi Output Value Register

#define	DIO32_GSC_D31LHOMR					DIO32_REG_ENCODE(4,0x5E0)// D31 Low-Hi Output Mask Register
#define	DIO32_GSC_D31LHOTSR					DIO32_REG_ENCODE(4,0x5E4)// D31 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D31LHOVR					DIO32_REG_ENCODE(4,0x5E8)// D31 Low-Hi Output Value Register

#define	DIO32_GSC_D31HLOMR					DIO32_REG_ENCODE(4,0x5F0)// D31 Low-Hi Output Mask Register
#define	DIO32_GSC_D31HLOTSR					DIO32_REG_ENCODE(4,0x5F4)// D31 Low-Hi Output Tri-State Register
#define	DIO32_GSC_D31HLOVR					DIO32_REG_ENCODE(4,0x5F8)// D31 Low-Hi Output Value Register



//*****************************************************************************
// DIO32_IOCTL_QUERY
//
//	Parameter:	s32
//		Pass in a value from the list below.
//		The value returned is the answer to the query.

typedef enum
{
	DIO32_QUERY_BOARD_INDEX,		//				Zero based index of the board.
	DIO32_QUERY_BUS_WIDTH,			//				32-bit or 64-bit
	DIO32_QUERY_COUNT,				//				How many query options are supported?
	DIO32_QUERY_DEVICE_TYPE,		// PCI ID regs	Value from gsc_dev_type_t
	DIO32_QUERY_FORM_FACTOR,		//				PCI, PMC, ...
	DIO32_QUERY_IO_BITS,			// BSR D16-D30
	DIO32_QUERY_USER_JUMPER_QTY,	// BSR D0-D3	How many user jumpers are supported?
	DIO32_QUERY_USER_JUMPER_SENSE,	// BSR D0-D3	What bit value means "jumper installed"?
	DIO32_QUERY_USER_JUMPER_VALUE,	// BSR D0-D3	The value read from the device.
	DIO32_QUERY_XCVR_TYPE,			//				If known.

	DIO32_IOCTL_QUERY_LAST

} dio32_query_t;

#define	DIO32_IOCTL_QUERY_ERROR				(-1)

// Options for DIO32_QUERY_FORM_FACTOR.
#define	DIO32_QUERY_FF_UNKNOWN				0
#define	DIO32_QUERY_FF_PCI					1
#define	DIO32_QUERY_FF_PMC					2
#define	DIO32_QUERY_FF_CPCI					3
#define	DIO32_QUERY_FF_PC104P				4
#define	DIO32_QUERY_FF_PCIE					5

// Options for DIO32_QUERY_XCVR_TYPE.
#define	DIO32_QUERY_XCVR_UNKNOWN			-1
#define	DIO32_QUERY_XCVR_RS485				0

//*****************************************************************************
// DIO32_IOCTL_INITIALIZE					BCR D31
//
//	Parameter:	None

//*****************************************************************************
// DIO32_IOCTL_DEGLITCH_FILTER				DGCR D0-D7
//
//	Parameter:	s32*
//		Valid values are from zero to 0xFF, and -1. Pass in -1 to retrieve
//		the current setting.

//*****************************************************************************
// DIO32_IOCTL_DEGLITCH_RATE				DGCR D8-D31
//
//	Parameter:	s32*
//		Valid values are from zero to 0xFFFFFF, and -1. Pass in -1 to retrieve
//		the current setting.

//*****************************************************************************
// DIO32_IOCTL_HL_OUT_MASK					DXXHLOMR D0-D31
// DIO32_IOCTL_LH_OUT_MASK					DXXLHOMR D0-D31
//
//	Parameter:	dio32_port_t*
//		Each bit applies to the corresponding cable signal number.
//		dio32_port_t.port	-1 for all supported, or index of specific port.
//		dio32_port_t.action	-1 to read, 0 to modify, 1 to write
//		dio32_port_t.value	Valid values are from 0 to 0xFFFFFFFF.
//		dio32_port_t.mask	Must be 0 for read or write, 0 to 0xFFFFFFFF for mod.

//*****************************************************************************
// DIO32_IOCTL_HL_OUT_TRI_STATE				DXXHLTSR D0-D31
// DIO32_IOCTL_LH_OUT_TRI_STATE				DXXLHTSR D0-D31
//
//	Parameter:	dio32_port_t*
//		Each bit applies to the corresponding cable signal number.
//		dio32_port_t.port	-1 for all supported, or index of specific port.
//		dio32_port_t.action	-1 to read, 0 to modify, 1 to write
//		dio32_port_t.value	Valid values are from 0 to 0xFFFFFFFF.
//		dio32_port_t.mask	Must be 0 for read or write, 0 to 0xFFFFFFFF for mod.

//*****************************************************************************
// DIO32_IOCTL_HL_OUT_VALUE					DXXHLOVR D0-D31
// DIO32_IOCTL_LH_OUT_VALUE					DXXLHOVR D0-D31
//
//	Parameter:	dio32_port_t*
//		Each bit applies to the corresponding cable signal number.
//		dio32_port_t.port	-1 for all supported, or index of specific port.
//		dio32_port_t.action	-1 to read, 0 to modify, 1 to write
//		dio32_port_t.value	Valid values are from 0 to 0xFFFFFFFF.
//		dio32_port_t.mask	Must be 0 for read or write, 0 to 0xFFFFFFFF for mod.

//*****************************************************************************
// DIO32_IOCTL_IO_IN_DEGLITCHED_GET			DGIR D0-D31
//
//	Parameter:	u32*
//		Valid values are from 0 to 0xFFFFFFFF.

//*****************************************************************************
// DIO32_IOCTL_IO_IN_GET					PIR D0-D31
//
//	Parameter:	u32*
//		Valid values are from 0 to 0xFFFFFFFF.

//*****************************************************************************
// DIO32_IOCTL_IO_OUT_CLOCK					OCR D0-D1, lower pair of each nibble
//
//	Parameter:	dio32_port_t*
//		Each bit applies to the corresponding cable signal number.
//		dio32_port_t.port	-1 for all supported, or index of specific port.
//		dio32_port_t.action	-1 to read, 0 to modify, 1 to write
//		dio32_port_t.value	A valid option from the below list.
//		dio32_port_t.mask	Must be 0 for read or write, 0 to 0xFFFFFFFF for mod.
#define	DIO32_IO_OUT_CLOCK_NONE				0
#define	DIO32_IO_OUT_CLOCK_1				1
#define	DIO32_IO_OUT_CLOCK_2				2
#define	DIO32_IO_OUT_CLOCK_3				3

//*****************************************************************************
// DIO32_IOCTL_IO_OUT_DIR					IODR D0-D31
//
//	Parameter:	dio32_port_t*
//		Each bit applies to the corresponding cable signal number.
//		dio32_port_t.port	-1 for all supported, or index of specific port.
//		dio32_port_t.action	-1 to read, 0 to modify, 1 to write
//		dio32_port_t.value	Valid values are from 0 to 0xFFFFFFFF.
//		dio32_port_t.mask	Must be 0 for read or write, 0 to 0xFFFFFFFF for mod.

//*****************************************************************************
// DIO32_IOCTL_IO_OUT_TRI_STATE				OTSR D0-D31
//
//	Parameter:	dio32_port_t*
//		Each bit applies to the corresponding cable signal number.
//		dio32_port_t.port	-1 for all supported, or index of specific port.
//		dio32_port_t.action	-1 to read, 0 to modify, 1 to write
//		dio32_port_t.value	Valid values are from 0 to 0xFFFFFFFF.
//		dio32_port_t.mask	Must be 0 for read or write, 0 to 0xFFFFFFFF for mod.

//*****************************************************************************
// DIO32_IOCTL_IO_OUT_VAL					OVR D0-D31
//
//	Parameter:	dio32_port_t*
//		Each bit applies to the corresponding cable signal number.
//		dio32_port_t.port	-1 for all supported, or index of specific port.
//		dio32_port_t.action	-1 to read, 0 to modify, 1 to write
//		dio32_port_t.value	Valid values are from 0 to 0xFFFFFFFF.
//		dio32_port_t.mask	Must be 0 for read or write, 0 to 0xFFFFFFFF for mod.

//*****************************************************************************
// DIO32_IOCTL_IRQ_HL_ENABLE				HLIER D0-D31
// DIO32_IOCTL_IRQ_LH_ENABLE				LHIER D0-D31
//
//	Parameter:	dio32_port_t*
//		Each bit applies to the corresponding cable signal number.
//		dio32_port_t.port	-1 for all supported, or index of specific port.
//		dio32_port_t.action	-1 to read, 0 to modify, 1 to write
//		dio32_port_t.value	Valid values are from 0 to 0xFFFFFFFF.
//		dio32_port_t.mask	Must be 0 for read or write, 0 to 0xFFFFFFFF for mod.

//*****************************************************************************
// DIO32_IOCTL_LED0							BCR D0-D1
// DIO32_IOCTL_LED1							BCR D2-D3
// DIO32_IOCTL_LED2							BCR D4-D5
// DIO32_IOCTL_LED3							BCR D6-D7
// DIO32_IOCTL_LED4							BCR D8-D9
// DIO32_IOCTL_LED5							BCR D10-D11
//
//	Parameter:	s32*
//		Pass in any combination of the below options, or
//		-1 to read the current setting.
#define	DIO32_LED_OFF						0
#define	DIO32_LED_GREEN						1
#define	DIO32_LED_RED						2
#define	DIO32_LED_BOTH						3

//*****************************************************************************
// DIO32_IOCTL_WAIT_EVENT		all fields must be valid
// DIO32_IOCTL_WAIT_CANCEL		fields need not be valid
// DIO32_IOCTL_WAIT_STATUS		fields need not be valid
//
//	Parameter:	dio32_wait_t*
// dio32_wait_t.flags - see gsc_wait_t.flags in gsc_common.h
// dio32_wait_t.main - see gsc_wait_t.main in gsc_common.h
// dio32_wait_t.gsc - Must be a combination of any or all of the below.
#define	DIO32_WAIT_GSC_LOW_HI				0x1		// ITR D0
#define	DIO32_WAIT_GSC_HI_LOW				0x2		// ITR D1
#define	DIO32_WAIT_GSC_ALL					0x3
// dio32_wait_t.hi_low - Valid values are from 0 to 0xFFFFFFFF, with each bit
//							representing the 32 Hi to Low interrupt sources.
#define	DIO32_WAIT_HI_LOW_ALL				0xFFFFFFFF
// dio32_wait_t.low_hi - Valid values are from 0 to 0xFFFFFFFF, with each bit
//							representing the 32 Low to Hi interrupt sources.
#define	DIO32_WAIT_LOW_HI_ALL				0xFFFFFFFF
// dio32_wait_t.timeout_ms - see gsc_wait_t.timeout_ms in gsc_common.h
// dio32_wait_t.count - see gsc_wait_t.count in gsc_common.h



#endif
