// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO32/DIO32_Linux_1.x_GSC_DN/samples/bbtest/gpio.c $
// $Rev: 54531 $
// $Date: 2024-05-28 09:31:25 -0500 (Tue, 28 May 2024) $

// DIO32: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _walking_0(const dio_t* dta, const dio_t* dtb)
{
	int	errs	= 0;
	u32	get;
	int	i;
	int	rx;
	u32	set;
	int	tx;
	int	xfer	= 0;

	gsc_label("Walking 0");

	// Initialize both devices/
	errs	+= dio32_initialize(dta->fd, -1, 0);
	errs	+= dio32_initialize(dtb->fd, -1, 0);

	// Configure the first device as all outputs.
	// Leave the second device as all inputs.
	errs	+= dio32_io_out_dir(dta->fd, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, 0xFFFFFFFF, NULL);

	// Walk the value.

	for (i = 0; (errs == 0) && (i <= 31); i++)
	{
		set		= ~(1 << i);
		tx		= dio32_io_out_val	(dta->fd, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, set, NULL);
		os_sleep_ms(10);
		rx		= dio32_io_in_get	(dtb->fd, -1, 0, &get);
		xfer	+= tx + rx;

		if ((tx == 0) && (rx == 0) && (get != set))
		{
			xfer++;
			printf(	"ERROR <---  (output 0x%08lX, but read 0x%08lX instead)\n",
					(unsigned long) set,
					(unsigned long) get);
		}
	}

	errs	+= xfer;

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}



//*****************************************************************************
static int _walking_1(const dio_t* dta, const dio_t* dtb)
{
	int	errs	= 0;
	u32	get;
	int	i;
	int	rx;
	u32	set;
	int	tx;
	int	xfer	= 0;

	gsc_label("Walking 1");

	// Initialize both devices/
	errs	+= dio32_initialize(dta->fd, -1, 0);
	errs	+= dio32_initialize(dtb->fd, -1, 0);

	// Configure the first device as all outputs.
	// Leave the second device as all inputs.
	errs	+= dio32_io_out_dir(dta->fd, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, 0xFFFFFFFF, NULL);

	// Walk the value.

	for (i = 0; (errs == 0) && (i <= 31); i++)
	{
		set		= 1 << i;
		tx		= dio32_io_out_val	(dta->fd, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, set, NULL);
		os_sleep_ms(10);
		rx		= dio32_io_in_get	(dtb->fd, -1, 0, &get);
		xfer	+= tx + rx;

		if ((tx == 0) && (rx == 0) && (get != set))
		{
			xfer++;
			printf(	"ERROR <---  (output 0x%08lX, but read 0x%08lX instead)\n",
					(unsigned long) set,
					(unsigned long) get);
		}
	}

	errs	+= xfer;

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}



//*****************************************************************************
static int _tests(const dio_t* dta, const dio_t* dtb)
{
	char	buf[64];
	int		errs	= 0;

	if (dta->index == dtb->index)
		sprintf(buf, "GPIO Tests (#%d, loopback)", dta->index);
	else
		sprintf(buf, "GPIO Tests (#%d => #%d)", dta->index, dtb->index);

	gsc_label(buf);
	printf("\n");
	gsc_label_level_inc();

	errs	+= _walking_0(dta, dtb);
	errs	+= _walking_1(dta, dtb);

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
int gpio_tests(const dio_t* dta, const dio_t* dtb)
{
	int	errs	= 0;

	if (dta->index == dtb->index)
	{
		errs	+= _tests(dta, dtb);
	}
	else
	{
		errs	+= _tests(dta, dtb);
		errs	+= _tests(dtb, dta);
	}

	return(errs);
}


