// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO32/DIO32_Linux_1.x_GSC_DN/samples/sbtest/input_response.c $
// $Rev: 51807 $
// $Date: 2022-10-14 12:34:33 -0500 (Fri, 14 Oct 2022) $

// DIO32: Sample Application: source file

#include "main.h"



// data types *****************************************************************

typedef struct
{
	const u32*	reg_list;
} _reg_list_t;

// variables ******************************************************************

static const _reg_list_t	_reg_list[6]	=
{
	{ hlomr_reg_list	},
	{ hlotsr_reg_list	},
	{ hlovr_reg_list	},
	{ lhomr_reg_list	},
	{ lhotsr_reg_list	},
	{ lhovr_reg_list	}
};



//*****************************************************************************
static int _verify_tx_access(int fd, int cmd, int port, int reg)
{
	dio32_port_t	arg;
	int				errs	= 0;
	int				i;
	int				j;
	const char*		name	= dio32_reg_get_name(reg);
	u32				pattern	= 0xA55AA55A;
	u32				val;

	// Write to a single port.
	errs		+= dio32_initialize(fd, -1, 0);
	arg.port	= port;
	arg.action	= DIO32_ACT_TX;
	arg.value	= pattern;
	arg.mask	= 0xFFFFFFFF;
	errs		+= dio32_ioctl_util(fd, -1, 0, cmd, &arg);

	// Verify that the command accessed the correct register.
	for (i = 0; i <= 5; i++)
	{
		for (j = 0; j <= 31; j++)
		{
			errs	+= dio32_reg_read(fd, -1, 0, _reg_list[i].reg_list[j], &val);

			if (_reg_list[i].reg_list[j] == reg)
			{
				if (val != pattern)
				{
					errs++;
					printf(	"FAIL <---  (%d. %s wrong value: "
							"got 0x%08lX, expected 0x%08lX)\n",
							__LINE__,
							name,
							(long) val,
							(long) pattern);
				}
				else if (j != port)
				{
					errs++;
					printf(	"FAIL <---  (%d. %s wrong port: "
							"accessed %ld, expected %ld)\n",
							__LINE__,
							name,
							(long) j,
							(long) port);
				}
			}
			else if (val != 0)
			{
				errs++;
				printf(	"FAIL <---  (%d. %s wrong value: "
						"got 0x%08lX, expected 0x%08lX)\n",
						__LINE__,
						name,
						(long) val,
						(long) 0);
			}
		}
	}

	errs	+= dio32_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _verify_rx_access(int fd, int cmd, int port, int reg)
{
	dio32_port_t	arg;
	int				errs	= 0;
	int				i;
	int				j;
	int				list	= 0;
	const char*		name;
	u32				pattern	= 0xA55AA55A;

	// Write to a single port.
	errs	+= dio32_initialize(fd, -1, 0);
	errs	+= dio32_reg_write(fd, -1, 0, reg, pattern);

	// Find out which list the register is in.

	for (i = 0; i <= 5; i++)
	{
		for (j = 0; j <= 31; j++)
		{
			if (reg == _reg_list[i].reg_list[j])
			{
				list	= i;
				break;
			}
		}
	}

	// Verify that the command reads the correct value.

	for (j = 0; j <= 31; j++)
	{
		name		= dio32_reg_get_name(_reg_list[list].reg_list[j]);
		arg.port	= j;
		arg.action	= DIO32_ACT_RX;
		arg.value	= 0;
		arg.mask	= 0xFFFFFFFF;
		errs		+= dio32_ioctl_util(fd, -1, 0, cmd, &arg);

		if (_reg_list[list].reg_list[j] == reg)
		{
			if (arg.value != pattern)
			{
				errs++;
				printf(	"FAIL <---  (%d. %s wrong value: "
						"got 0x%08lX, expected 0x%08lX)\n",
						__LINE__,
						name,
						(long) arg.value,
						(long) pattern);
			}
			else if (j != port)
			{
				errs++;
				printf(	"FAIL <---  (%d. %s wrong port: "
						"accessed %ld, expected %ld)\n",
						__LINE__,
						name,
						(long) j,
						(long) port);
			}
		}
		else if (arg.value != 0)
		{
			errs++;
			printf(	"FAIL <---  (%d. %s wrong value: "
					"got 0x%08lX, expected 0x%08lX)\n",
					__LINE__,
					name,
					(long) arg.value,
					(long) 0);
		}
	}

	errs	+= dio32_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _verify_mx_access(int fd, int cmd, int port, int reg)
{
	dio32_port_t	arg;
	int				errs	= 0;
	int				i;
	int				j;
	int				k;
	const char*		name	= dio32_reg_get_name(reg);
	u32				pattern	= 0xA55AA55A;
	u32				val;

	// Walking one: set via service with single bit set

	for (k = 0; k <= 31; k++)
	{
		// Write to a single port.
		errs		+= dio32_initialize(fd, -1, 0);
		pattern		= 1 << k;

		arg.port	= port;
		arg.action	= DIO32_ACT_MX;
		arg.value	= pattern;
		arg.mask	= 0xFFFFFFFF;
		errs		+= dio32_ioctl_util(fd, -1, 0, cmd, &arg);

		// Verify that the command accessed the correct register.
		for (i = 0; i <= 5; i++)
		{
			for (j = 0; j <= 31; j++)
			{
				errs	+= dio32_reg_read(fd, -1, 0, _reg_list[i].reg_list[j], &val);

				if (_reg_list[i].reg_list[j] == reg)
				{
					if (val != pattern)
					{
						errs++;
						printf(	"FAIL <---  (%d. %s wrong value: "
								"got 0x%08lX, expected 0x%08lX)\n",
								__LINE__,
								name,
								(long) val,
								(long) pattern);
					}
					else if (j != port)
					{
						errs++;
						printf(	"FAIL <---  (%d. %s wrong port: "
								"accessed %ld, expected %ld)\n",
								__LINE__,
								name,
								(long) j,
								(long) port);
					}
				}
				else if (val != 0)
				{
					errs++;
					printf(	"FAIL <---  (%d. %s wrong value: "
							"got 0x%08lX, expected 0x%08lX)\n",
							__LINE__,
							name,
							(long) val,
							(long) 0);
				}
			}
		}
	}

	// Walking one: set via service with single bit mask

	for (k = 0; k <= 31; k++)
	{
		// Write to a single port.
		errs		+= dio32_initialize(fd, -1, 0);
		pattern		= 1 << k;

		arg.port	= port;
		arg.action	= DIO32_ACT_MX;
		arg.value	= 0xFFFFFFFF;
		arg.mask	= pattern;
		errs		+= dio32_ioctl_util(fd, -1, 0, cmd, &arg);

		// Verify that the command accessed the correct register.
		for (i = 0; i <= 5; i++)
		{
			for (j = 0; j <= 31; j++)
			{
				errs	+= dio32_reg_read(fd, -1, 0, _reg_list[i].reg_list[j], &val);

				if (_reg_list[i].reg_list[j] == reg)
				{
					if (val != pattern)
					{
						errs++;
						printf(	"FAIL <---  (%d. %s wrong value: "
								"got 0x%08lX, expected 0x%08lX)\n",
								__LINE__,
								name,
								(long) val,
								(long) pattern);
					}
					else if (j != port)
					{
						errs++;
						printf(	"FAIL <---  (%d. %s wrong port: "
								"accessed %ld, expected %ld)\n",
								__LINE__,
								name,
								(long) j,
								(long) port);
					}
				}
				else if (val != 0)
				{
					errs++;
					printf(	"FAIL <---  (%d. %s wrong value: "
							"got 0x%08lX, expected 0x%08lX)\n",
							__LINE__,
							name,
							(long) val,
							(long) 0);
				}
			}
		}
	}

	errs	+= dio32_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _verify_errors(int fd, int cmd, int port, int reg)
{
	dio32_port_t	arg;
	int				errs	= 0;
	int				ret;

	// Verify that errors are reported.
	errs	+= dio32_initialize(fd, -1, 0);

	// valid:
	arg.port	= port;
	arg.action	= DIO32_ACT_TX;
	arg.value	= 0;
	arg.mask	= 0xFFFFFFFF;
	ret			= dio32_ioctl(fd, cmd, &arg);

	if (ret)
	{
		errs++;
		printf("FAIL <---  (%d. failure reported, but shouldn't)\n", __LINE__);
	}

	// invalid: port
	arg.port	= -1;
	arg.action	= DIO32_ACT_TX;
	arg.value	= 0;
	arg.mask	= 0xFFFFFFFF;
	ret			= dio32_ioctl(fd, cmd, &arg);

	if (ret == 0)
	{
		errs++;
		printf("FAIL <---  (%d. failure not reported: port)\n", __LINE__);
	}

	// invalid: port
	arg.port	= 33;
	arg.action	= DIO32_ACT_TX;
	arg.value	= 0;
	arg.mask	= 0xFFFFFFFF;
	ret			= dio32_ioctl(fd, cmd, &arg);

	if (ret == 0)
	{
		errs++;
		printf("FAIL <---  (%d. failure not reported: port)\n", __LINE__);
	}

	// invalid: action
	arg.port	= 0;
	arg.action	= -1;
	arg.value	= 0;
	arg.mask	= 0xFFFFFFFF;
	ret			= dio32_ioctl(fd, cmd, &arg);

	if (ret == 0)
	{
		errs++;
		printf("FAIL <---  (%d. failure not reported: action)\n", __LINE__);
	}

	// invalid: action
	arg.port	= -1;
	arg.action	= 4;
	arg.value	= 0;
	arg.mask	= 0xFFFFFFFF;
	ret			= dio32_ioctl(fd, cmd, &arg);

	if (ret == 0)
	{
		errs++;
		printf("FAIL <---  (%d. failure not reported: action)\n", __LINE__);
	}

	errs	+= dio32_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	input_response_service_test
*
*	Purpose:
*
*		Verify that the specified service interacts with the device as expected.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*		cmd		The IOCTL command code for the service to be tested.
*
*		port	The index of the port to access: 0 to 31
*
*		reg		The register the service directly accesses.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int input_response_service_test(int fd, int cmd, int port, int reg)
{
	int	errs	= 0;

	errs	+= _verify_tx_access(fd, cmd, port, reg);
	errs	+= _verify_rx_access(fd, cmd, port, reg);
	errs	+= _verify_mx_access(fd, cmd, port, reg);
	errs	+= _verify_errors(fd, cmd, port, reg);

	return(errs);
}



// variables ******************************************************************

const u32	hlomr_reg_list[]	=
{
	DIO32_GSC_D00HLOMR,
	DIO32_GSC_D01HLOMR,
	DIO32_GSC_D02HLOMR,
	DIO32_GSC_D03HLOMR,
	DIO32_GSC_D04HLOMR,
	DIO32_GSC_D05HLOMR,
	DIO32_GSC_D06HLOMR,
	DIO32_GSC_D07HLOMR,
	DIO32_GSC_D08HLOMR,
	DIO32_GSC_D09HLOMR,
	DIO32_GSC_D10HLOMR,
	DIO32_GSC_D11HLOMR,
	DIO32_GSC_D12HLOMR,
	DIO32_GSC_D13HLOMR,
	DIO32_GSC_D14HLOMR,
	DIO32_GSC_D15HLOMR,
	DIO32_GSC_D16HLOMR,
	DIO32_GSC_D17HLOMR,
	DIO32_GSC_D18HLOMR,
	DIO32_GSC_D19HLOMR,
	DIO32_GSC_D20HLOMR,
	DIO32_GSC_D21HLOMR,
	DIO32_GSC_D22HLOMR,
	DIO32_GSC_D23HLOMR,
	DIO32_GSC_D24HLOMR,
	DIO32_GSC_D25HLOMR,
	DIO32_GSC_D26HLOMR,
	DIO32_GSC_D27HLOMR,
	DIO32_GSC_D28HLOMR,
	DIO32_GSC_D29HLOMR,
	DIO32_GSC_D30HLOMR,
	DIO32_GSC_D31HLOMR
};

const u32	hlotsr_reg_list[]	=
{
	DIO32_GSC_D00HLOTSR,
	DIO32_GSC_D01HLOTSR,
	DIO32_GSC_D02HLOTSR,
	DIO32_GSC_D03HLOTSR,
	DIO32_GSC_D04HLOTSR,
	DIO32_GSC_D05HLOTSR,
	DIO32_GSC_D06HLOTSR,
	DIO32_GSC_D07HLOTSR,
	DIO32_GSC_D08HLOTSR,
	DIO32_GSC_D09HLOTSR,
	DIO32_GSC_D10HLOTSR,
	DIO32_GSC_D11HLOTSR,
	DIO32_GSC_D12HLOTSR,
	DIO32_GSC_D13HLOTSR,
	DIO32_GSC_D14HLOTSR,
	DIO32_GSC_D15HLOTSR,
	DIO32_GSC_D16HLOTSR,
	DIO32_GSC_D17HLOTSR,
	DIO32_GSC_D18HLOTSR,
	DIO32_GSC_D19HLOTSR,
	DIO32_GSC_D20HLOTSR,
	DIO32_GSC_D21HLOTSR,
	DIO32_GSC_D22HLOTSR,
	DIO32_GSC_D23HLOTSR,
	DIO32_GSC_D24HLOTSR,
	DIO32_GSC_D25HLOTSR,
	DIO32_GSC_D26HLOTSR,
	DIO32_GSC_D27HLOTSR,
	DIO32_GSC_D28HLOTSR,
	DIO32_GSC_D29HLOTSR,
	DIO32_GSC_D30HLOTSR,
	DIO32_GSC_D31HLOTSR
};



const u32	hlovr_reg_list[]	=
{
	DIO32_GSC_D00HLOVR,
	DIO32_GSC_D01HLOVR,
	DIO32_GSC_D02HLOVR,
	DIO32_GSC_D03HLOVR,
	DIO32_GSC_D04HLOVR,
	DIO32_GSC_D05HLOVR,
	DIO32_GSC_D06HLOVR,
	DIO32_GSC_D07HLOVR,
	DIO32_GSC_D08HLOVR,
	DIO32_GSC_D09HLOVR,
	DIO32_GSC_D10HLOVR,
	DIO32_GSC_D11HLOVR,
	DIO32_GSC_D12HLOVR,
	DIO32_GSC_D13HLOVR,
	DIO32_GSC_D14HLOVR,
	DIO32_GSC_D15HLOVR,
	DIO32_GSC_D16HLOVR,
	DIO32_GSC_D17HLOVR,
	DIO32_GSC_D18HLOVR,
	DIO32_GSC_D19HLOVR,
	DIO32_GSC_D20HLOVR,
	DIO32_GSC_D21HLOVR,
	DIO32_GSC_D22HLOVR,
	DIO32_GSC_D23HLOVR,
	DIO32_GSC_D24HLOVR,
	DIO32_GSC_D25HLOVR,
	DIO32_GSC_D26HLOVR,
	DIO32_GSC_D27HLOVR,
	DIO32_GSC_D28HLOVR,
	DIO32_GSC_D29HLOVR,
	DIO32_GSC_D30HLOVR,
	DIO32_GSC_D31HLOVR
};



const u32	lhomr_reg_list[32]	=
{
	DIO32_GSC_D00LHOMR,
	DIO32_GSC_D01LHOMR,
	DIO32_GSC_D02LHOMR,
	DIO32_GSC_D03LHOMR,
	DIO32_GSC_D04LHOMR,
	DIO32_GSC_D05LHOMR,
	DIO32_GSC_D06LHOMR,
	DIO32_GSC_D07LHOMR,
	DIO32_GSC_D08LHOMR,
	DIO32_GSC_D09LHOMR,
	DIO32_GSC_D10LHOMR,
	DIO32_GSC_D11LHOMR,
	DIO32_GSC_D12LHOMR,
	DIO32_GSC_D13LHOMR,
	DIO32_GSC_D14LHOMR,
	DIO32_GSC_D15LHOMR,
	DIO32_GSC_D16LHOMR,
	DIO32_GSC_D17LHOMR,
	DIO32_GSC_D18LHOMR,
	DIO32_GSC_D19LHOMR,
	DIO32_GSC_D20LHOMR,
	DIO32_GSC_D21LHOMR,
	DIO32_GSC_D22LHOMR,
	DIO32_GSC_D23LHOMR,
	DIO32_GSC_D24LHOMR,
	DIO32_GSC_D25LHOMR,
	DIO32_GSC_D26LHOMR,
	DIO32_GSC_D27LHOMR,
	DIO32_GSC_D28LHOMR,
	DIO32_GSC_D29LHOMR,
	DIO32_GSC_D30LHOMR,
	DIO32_GSC_D31LHOMR
};



const u32	lhotsr_reg_list[]	=
{
	DIO32_GSC_D00LHOTSR,
	DIO32_GSC_D01LHOTSR,
	DIO32_GSC_D02LHOTSR,
	DIO32_GSC_D03LHOTSR,
	DIO32_GSC_D04LHOTSR,
	DIO32_GSC_D05LHOTSR,
	DIO32_GSC_D06LHOTSR,
	DIO32_GSC_D07LHOTSR,
	DIO32_GSC_D08LHOTSR,
	DIO32_GSC_D09LHOTSR,
	DIO32_GSC_D10LHOTSR,
	DIO32_GSC_D11LHOTSR,
	DIO32_GSC_D12LHOTSR,
	DIO32_GSC_D13LHOTSR,
	DIO32_GSC_D14LHOTSR,
	DIO32_GSC_D15LHOTSR,
	DIO32_GSC_D16LHOTSR,
	DIO32_GSC_D17LHOTSR,
	DIO32_GSC_D18LHOTSR,
	DIO32_GSC_D19LHOTSR,
	DIO32_GSC_D20LHOTSR,
	DIO32_GSC_D21LHOTSR,
	DIO32_GSC_D22LHOTSR,
	DIO32_GSC_D23LHOTSR,
	DIO32_GSC_D24LHOTSR,
	DIO32_GSC_D25LHOTSR,
	DIO32_GSC_D26LHOTSR,
	DIO32_GSC_D27LHOTSR,
	DIO32_GSC_D28LHOTSR,
	DIO32_GSC_D29LHOTSR,
	DIO32_GSC_D30LHOTSR,
	DIO32_GSC_D31LHOTSR
};



const u32	lhovr_reg_list[]	=
{
	DIO32_GSC_D00LHOVR,
	DIO32_GSC_D01LHOVR,
	DIO32_GSC_D02LHOVR,
	DIO32_GSC_D03LHOVR,
	DIO32_GSC_D04LHOVR,
	DIO32_GSC_D05LHOVR,
	DIO32_GSC_D06LHOVR,
	DIO32_GSC_D07LHOVR,
	DIO32_GSC_D08LHOVR,
	DIO32_GSC_D09LHOVR,
	DIO32_GSC_D10LHOVR,
	DIO32_GSC_D11LHOVR,
	DIO32_GSC_D12LHOVR,
	DIO32_GSC_D13LHOVR,
	DIO32_GSC_D14LHOVR,
	DIO32_GSC_D15LHOVR,
	DIO32_GSC_D16LHOVR,
	DIO32_GSC_D17LHOVR,
	DIO32_GSC_D18LHOVR,
	DIO32_GSC_D19LHOVR,
	DIO32_GSC_D20LHOVR,
	DIO32_GSC_D21LHOVR,
	DIO32_GSC_D22LHOVR,
	DIO32_GSC_D23LHOVR,
	DIO32_GSC_D24LHOVR,
	DIO32_GSC_D25LHOVR,
	DIO32_GSC_D26LHOVR,
	DIO32_GSC_D27LHOVR,
	DIO32_GSC_D28LHOVR,
	DIO32_GSC_D29LHOVR,
	DIO32_GSC_D30LHOVR,
	DIO32_GSC_D31LHOVR
};


