// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO32/DIO32_Linux_1.x_GSC_DN/samples/sbtest/io_in_get.c $
// $Rev: 51807 $
// $Date: 2022-10-14 12:34:33 -0500 (Fri, 14 Oct 2022) $

// DIO32: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	// There are no setings we can test here.
	return(0);
}



//*****************************************************************************
static int _function_test(int fd)
{
	int errs	= 0;
	u32	get;
	int	i;
	u32	pir;
	u32	set;

	errs	+= dio32_initialize(fd, -1, 0);

	// Program the pins to outputs then drive
	// the outputs so we can monitor the inputs.
	errs	+= dio32_io_out_dir(fd, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, 0xFFFFFFFF, NULL);
	errs	+= dio32_io_out_tri_state(fd, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, 0, NULL);

	// Walking one.

	for (i = 0; i <= 31; i++)
	{
		set		= 1 << i;
		errs	+= dio32_io_out_val(fd, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, set, NULL);
		errs	+= dio32_io_in_get(fd, -1, 0, &get);
		errs	+= dio32_reg_read(fd, -1, 0, DIO32_GSC_PIR, &pir);

		if (get != set)
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. W1: Raw input error:"
					" expected 0x%08lX, got 0x%08lX)\n",
					__LINE__,
					(long) set,
					(long) get);
		}

		if (pir != get)
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. W1: PIR error:"
					" expected 0x%08lX, got 0x%08lX)\n",
					__LINE__,
					(long) get,
					(long) pir);
		}
	}

	// Walking zero.

	for (i = 0; i <= 31; i++)
	{
		set		= ~(1 << i);
		errs	+= dio32_io_out_val(fd, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, set, NULL);
		errs	+= dio32_io_in_get(fd, -1, 0, &get);
		errs	+= dio32_reg_read(fd, -1, 0, DIO32_GSC_PIR, &pir);

		if (get != set)
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. W0: Raw input error:"
					" expected 0x%08lX, got 0x%08lX)\n",
					__LINE__,
					(long) set,
					(long) get);
		}

		if (pir != get)
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. W0: PIR error:"
					" expected 0x%08lX, got 0x%08lX)\n",
					__LINE__,
					(long) get,
					(long) pir);
		}
	}

	errs	+= dio32_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	io_in_get_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service DIO32_IOCTL_IO_IN_GET.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int io_in_get_test(int fd)
{
	int	errs	= 0;

	gsc_label("DIO32_IOCTL_IO_IN_GET");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


