// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO32/DIO32_Linux_1.x_GSC_DN/samples/sbtest/io_out_clock.c $
// $Rev: 51807 $
// $Date: 2022-10-14 12:34:33 -0500 (Fri, 14 Oct 2022) $

// DIO32: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	static const u32	list[4] =
	{
		DIO32_IO_OUT_CLOCK_NONE,
		DIO32_IO_OUT_CLOCK_1,
		DIO32_IO_OUT_CLOCK_2,
		DIO32_IO_OUT_CLOCK_3
	};

	dio32_port_t	arg;
	int				errs	= 0;
	u32				get;
	int				i;
	int				j;
	u32				ocr;
	int				ret;
	u32				set;
	u32				val;

	// Set via service

	for (i = 0; i <= 31; i += 4)
	{
		for (j = 0; j <= 3; j++)
		{
			set		= list[j];
			val		= set << i;
			errs	+= dio32_initialize(fd, -1, 0);
			errs	+= dio32_io_out_clock(fd, -1, 0, i, DIO32_ACT_TX, 0, set, NULL);
			errs	+= dio32_io_out_clock(fd, -1, 0, i, DIO32_ACT_RX, 0, 0, &get);
			errs	+= dio32_reg_read(fd, -1, 0, DIO32_GSC_OCR, &ocr);

			if (get != set)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. set error: "
						"expected 0x%08lX, got 0x%08lX)\n",
						__LINE__,
						(long) set,
						(long) get);
			}

			if (ocr != (set << i))
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. OCR 1 error: "
						"expected 0x%08lX, got 0x%08lX)\n",
						__LINE__,
						(long) set << i,
						(long) ocr);
			}

			if (ocr != val)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. set OCR 2 error:"
						" expected 0x%08lX, got 0x%08lX)\n",
						__LINE__,
						(long) ocr,
						(long) val);
			}
		}
	}

	// Set via register write

	for (i = 0; i <= 31; i += 4)
	{
		for (j = 0; j <= 3; j++)
		{
			set		= list[j];
			val		= set << i;
			errs	+= dio32_initialize(fd, -1, 0);
			errs	+= dio32_reg_write(fd, -1, 0, DIO32_GSC_OCR, val);
			errs	+= dio32_io_out_clock(fd, -1, 0, i, DIO32_ACT_RX, 0, 0, &get);

			if (val != (get << i))
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. OCR 1 error: "
						"expected 0x%08lX, got 0x%08lX)\n",
						__LINE__,
						(long) set << i,
						(long) ocr);
			}
		}
	}

	// Verify that errors are reported.
	errs	+= dio32_initialize(fd, -1, 0);

	// valid:
	arg.port	= -1;
	arg.action	= DIO32_ACT_RX;
	arg.value	= 0;
	arg.mask	= 0x11111111;
	ret			= dio32_ioctl(fd, DIO32_IOCTL_IO_OUT_CLOCK, &arg);

	if (ret)
	{
		errs++;
		printf("FAIL <---  (%d. failure reported, but shouldn't)\n", __LINE__);
	}

	// invalid: port
	arg.port	= -2;
	arg.action	= DIO32_ACT_TX;
	arg.value	= 0;
	arg.mask	= 0x11111111;
	ret			= dio32_ioctl(fd, DIO32_IOCTL_IO_OUT_CLOCK, &arg);

	if (ret == 0)
	{
		errs++;
		printf("FAIL <---  (%d. failure not reported: port)\n", __LINE__);
	}

	// invalid: port
	arg.port	= 33;
	arg.action	= DIO32_ACT_TX;
	arg.value	= 0;
	arg.mask	= 0x11111111;
	ret			= dio32_ioctl(fd, DIO32_IOCTL_IO_OUT_CLOCK, &arg);

	if (ret == 0)
	{
		errs	= 1;
		printf("FAIL <---  (%d. failure not reported: port)\n", __LINE__);
	}

	// invalid: MX is not supported
	arg.port	= -1;
	arg.action	= DIO32_ACT_MX;
	arg.value	= 0;
	arg.mask	= 0x11111111;
	ret			= dio32_ioctl(fd, DIO32_IOCTL_IO_OUT_CLOCK, &arg);

	if (ret == 0)
	{
		errs	= 1;
		printf("FAIL <---  (%d. failure not reported: MX)\n", __LINE__);
	}

	// invalid: action
	arg.port	= -1;
	arg.action	= -1;
	arg.value	= 0;
	arg.mask	= 0x11111111;
	ret			= dio32_ioctl(fd, DIO32_IOCTL_IO_OUT_CLOCK, &arg);

	if (ret == 0)
	{
		errs	= 1;
		printf("FAIL <---  (%d. failure not reported: action)\n", __LINE__);
	}

	// invalid: action
	arg.port	= -1;
	arg.action	= 4;
	arg.value	= 0;
	arg.mask	= 0x11111111;
	ret			= dio32_ioctl(fd, DIO32_IOCTL_IO_OUT_CLOCK, &arg);

	if (ret == 0)
	{
		errs	= 1;
		printf("FAIL <---  (%d. failure not reported: action)\n", __LINE__);
	}

	// invalid: value
	arg.port	= -1;
	arg.action	= DIO32_ACT_TX;
	arg.value	= 999;
	arg.mask	= 0x11111111;
	ret			= dio32_ioctl(fd, DIO32_IOCTL_IO_OUT_CLOCK, &arg);

	if (ret == 0)
	{
		errs++;
		printf("FAIL <---  (%d. failure not reported: value)\n", __LINE__);
	}

	for (j = 0; j <= 31; j++)
	{
		val	= 1 << j;

		// Test port options.
		arg.port	= j;
		arg.action	= DIO32_ACT_TX;
		arg.value	= DIO32_IO_OUT_CLOCK_NONE;
		arg.mask	= 0x11111111;
		ret			= dio32_ioctl(fd, DIO32_IOCTL_IO_OUT_CLOCK, &arg);

		if (val & 0x11111111)
		{
			// This option is valid.

			if (ret)
			{
				errs++;
				printf(	"FAIL <---  (%d. failure reported, but shouldn't: j %d)\n",
						__LINE__,
						j);
			}
		}
		else
		{
			// This option is invalid.

			if (ret == 0)
			{
				errs++;
				printf(	"FAIL <---  (%d. failure not reported: j %d)\n",
						__LINE__,
						j);
			}
		}

		// Test mask options.
		arg.port	= -1;
		arg.action	= DIO32_ACT_TX;
		arg.value	= DIO32_IO_OUT_CLOCK_NONE;
		arg.mask	= val;
		ret			= dio32_ioctl(fd, DIO32_IOCTL_IO_OUT_CLOCK, &arg);

		if (val & 0x11111111)
		{
			// This option is valid.

			if (ret)
			{
				errs++;
				printf(	"FAIL <---  (%d. failure reported, but shouldn't: j %d)\n",
						__LINE__,
						j);
			}
		}
		else
		{
			// This option is invalid.

			if (ret == 0)
			{
				errs++;
				printf(	"FAIL <---  (%d. failure not reported: j %d)\n",
						__LINE__,
						j);
			}
		}
	}

	errs	+= dio32_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	// We need additional equipment for this so we do nothing here.
	return(0);
}



/******************************************************************************
*
*	Function:	io_out_clock_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service DIO32_IOCTL_IO_OUT_CLOCK.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int io_out_clock_test(int fd)
{
	int	errs	= 0;

	gsc_label("DIO32_IOCTL_IO_OUT_CLOCK");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


