// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO32/DIO32_Linux_1.x_GSC_DN/samples/sbtest/io_out_dir.c $
// $Rev: 51807 $
// $Date: 2022-10-14 12:34:33 -0500 (Fri, 14 Oct 2022) $

// DIO32: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	dio32_port_t	arg;
	int				errs	= 0;
	u32				get;
	int				i;
	int				j;
	u32				iodr;
	u32				mask;
	int				ret;
	u32				set;

	// Walking one: set via service with single bit set
	errs	+= dio32_initialize(fd, -1, 0);

	for (i = 0; i <= 31; i++)
	{
		set	= 1 << i;
		errs	+= dio32_io_out_dir(fd, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, set, NULL);
		errs	+= dio32_io_out_dir(fd, -1, 0, -1, DIO32_ACT_RX, 0xFFFFFFFF, 0, &get);
		errs	+= dio32_reg_read(fd, -1, 0, DIO32_GSC_IODR, &iodr);

		if (get != set)
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. W1 set error: "
					"expected 0x%08lX, got 0x%08lX)\n",
					__LINE__,
					(long) set,
					(long) get);
		}

		if (iodr != get)
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. W1: IODR error:"
					" expected 0x%08lX, got 0x%08lX)\n",
					__LINE__,
					(long) get,
					(long) iodr);
		}
	}

	// Walking one: set via service with single bit mask
	errs	+= dio32_initialize(fd, -1, 0);

	for (i = 0; i <= 31; i++)
	{
		set		= 1 << i;
		errs	+= dio32_io_out_dir(fd, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, 0, NULL);
		errs	+= dio32_io_out_dir(fd, -1, 0, -1, DIO32_ACT_TX, set, 0xFFFFFFFF, NULL);
		errs	+= dio32_io_out_dir(fd, -1, 0, -1, DIO32_ACT_RX, 0xFFFFFFFF, 0, &get);
		errs	+= dio32_reg_read(fd, -1, 0, DIO32_GSC_IODR, &iodr);

		if (get != set)
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. W1 set error: "
					"expected 0x%08lX, got 0x%08lX)\n",
					__LINE__,
					(long) set,
					(long) get);
		}

		if (iodr != get)
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. W1: IODR error:"
					" expected 0x%08lX, got 0x%08lX)\n",
					__LINE__,
					(long) get,
					(long) iodr);
		}
	}

	// Walking one: set via register write
	errs	+= dio32_initialize(fd, -1, 0);

	for (i = 0; i <= 31; i++)
	{
		set	= 1 << i;
		errs	+= dio32_reg_write(fd, -1, 0, DIO32_GSC_IODR, set);
		errs	+= dio32_io_out_dir(fd, -1, 0, -1, DIO32_ACT_RX, 0xFFFFFFFF, 0, &get);

		if (get != set)
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. W1 get error: "
					"expected 0x%08lX, got 0x%08lX)\n",
					__LINE__,
					(long) set,
					(long) get);
		}
	}

	// Walking zero: set via service
	errs	+= dio32_initialize(fd, -1, 0);

	for (i = 0; i <= 31; i++)
	{
		set	= ~(1 << i);
		errs	+= dio32_io_out_dir(fd, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, set, NULL);
		errs	+= dio32_io_out_dir(fd, -1, 0, -1, DIO32_ACT_RX, 0xFFFFFFFF, 0, &get);
		errs	+= dio32_reg_read(fd, -1, 0, DIO32_GSC_IODR, &iodr);

		if (get != set)
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. W0 set error: "
					"expected 0x%08lX, got 0x%08lX)\n",
					__LINE__,
					(long) set,
					(long) get);
		}

		if (iodr != get)
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. W0: IODR error:"
					" expected 0x%08lX, got 0x%08lX)\n",
					__LINE__,
					(long) get,
					(long) iodr);
		}
	}

	// Walking zero: set via register write
	errs	+= dio32_initialize(fd, -1, 0);

	for (i = 0; i <= 31; i++)
	{
		set	= ~(1 << i);
		errs	+= dio32_reg_write(fd, -1, 0, DIO32_GSC_IODR, set);
		errs	+= dio32_io_out_dir(fd, -1, 0, -1, DIO32_ACT_RX, 0xFFFFFFFF, 0, &get);

		if (get != set)
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. W0 get error: "
					"expected 0x%08lX, got 0x%08lX)\n",
					__LINE__,
					(long) set,
					(long) get);
		}
	}

	// Walking one: read via secondary walking mask
	errs	+= dio32_initialize(fd, -1, 0);

	for (i = 0; i <= 31; i++)
	{
		set	= 1 << i;
		errs	+= dio32_reg_write(fd, -1, 0, DIO32_GSC_IODR, set);

		for (j = 0; j <= 31; j++)
		{
			mask	= 1 << j;
			errs	+= dio32_io_out_dir(fd, -1, 0, -1, DIO32_ACT_RX, mask, 0, &get);

			if (j == i)
			{
				if (get != set)
				{
					errs	= 1;
					printf(	"FAIL <---  (%d. W1 get error: "
							"expected 0x%08lX, got 0x%08lX)\n",
							__LINE__,
							(long) set,
							(long) get);
				}
			}
			else if (get)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. W1 get error: "
						"expected 0x%08lX, got 0x%08lX)\n",
						__LINE__,
						(long) 0,
						(long) get);
			}
		}
	}

	// Verify that errors are reported.
	errs	+= dio32_initialize(fd, -1, 0);

	// valid:
	arg.port	= -1;
	arg.action	= DIO32_ACT_TX;
	arg.value	= 0;
	arg.mask	= 0xFFFFFFFF;
	ret			= dio32_ioctl(fd, DIO32_IOCTL_IO_OUT_DIR, &arg);

	if (ret)
	{
		errs++;
		printf("FAIL <---  (%d. failure reported, but shouldn't)\n", __LINE__);
	}

	// invalid: port
	arg.port	= -2;
	arg.action	= DIO32_ACT_TX;
	arg.value	= 0;
	arg.mask	= 0xFFFFFFFF;
	ret			= dio32_ioctl(fd, DIO32_IOCTL_IO_OUT_DIR, &arg);

	if (ret == 0)
	{
		errs	= 1;
		printf("FAIL <---  (%d. failure not reported: port)\n", __LINE__);
	}

	// invalid: port
	arg.port	= 33;
	arg.action	= DIO32_ACT_TX;
	arg.value	= 0;
	arg.mask	= 0xFFFFFFFF;
	ret			= dio32_ioctl(fd, DIO32_IOCTL_IO_OUT_DIR, &arg);

	if (ret == 0)
	{
		errs++;
		printf("FAIL <---  (%d. failure not reported: port)\n", __LINE__);
	}

	// invalid: action
	arg.port	= -1;
	arg.action	= -1;
	arg.value	= 0;
	arg.mask	= 0xFFFFFFFF;
	ret			= dio32_ioctl(fd, DIO32_IOCTL_IO_OUT_DIR, &arg);

	if (ret == 0)
	{
		errs++;
		printf("FAIL <---  (%d. failure not reported: action)\n", __LINE__);
	}

	// invalid: action
	arg.port	= -1;
	arg.action	= 4;
	arg.value	= 0;
	arg.mask	= 0xFFFFFFFF;
	ret			= dio32_ioctl(fd, DIO32_IOCTL_IO_OUT_DIR, &arg);

	if (ret == 0)
	{
		errs	= 1;
		printf("FAIL <---  (%d. failure not reported: action)\n", __LINE__);
	}

	errs	+= dio32_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	// We need additional equipment for this so we do nothing here.
	return(0);
}



/******************************************************************************
*
*	Function:	io_out_dir_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service DIO32_IOCTL_IO_OUT_DIR.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int io_out_dir_test(int fd)
{
	int	errs	= 0;

	gsc_label("DIO32_IOCTL_IO_OUT_DIR");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


