// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO32/DIO32_Linux_1.x_GSC_DN/samples/sbtest/led3.c $
// $Rev: 54538 $
// $Date: 2024-05-28 09:36:47 -0500 (Tue, 28 May 2024) $

// DIO32: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* line		*/	__LINE__,
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	DIO32_IOCTL_LED3,
			/* arg		*/	DIO32_LED_OFF,
			/* reg		*/	DIO32_GSC_BCR,
			/* mask		*/	0x0C0,
			/* value	*/	0x000
		},
		{
			/* line		*/	__LINE__,
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	DIO32_IOCTL_LED3,
			/* arg		*/	DIO32_LED_GREEN,
			/* reg		*/	DIO32_GSC_BCR,
			/* mask		*/	0x0C0,
			/* value	*/	0x040
		},
		{
			/* line		*/	__LINE__,
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	DIO32_IOCTL_LED3,
			/* arg		*/	DIO32_LED_RED,
			/* reg		*/	DIO32_GSC_BCR,
			/* mask		*/	0x0C0,
			/* value	*/	0x080
		},
		{
			/* line		*/	__LINE__,
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	DIO32_IOCTL_LED3,
			/* arg		*/	DIO32_LED_BOTH,
			/* reg		*/	DIO32_GSC_BCR,
			/* mask		*/	0x0C0,
			/* value	*/	0x0C0
		},

		{ 0, SERVICE_END_LIST }
	};

	int errs	= 0;
	int	ret;
	s32	val;

	errs	+= dio32_initialize(fd, -1, 0);

	errs	+= service_ioctl_reg_get_list(fd, list);
	errs	+= service_ioctl_set_reg_list(fd, list);

	errs	+= service_ioctl_reg_get_list(fd, list);
	errs	+= service_ioctl_set_reg_list(fd, list);

	// Verify that errors are reported.
	errs	+= dio32_initialize(fd, -1, 0);

	// valid:
	val	= DIO32_LED_OFF;
	ret	= dio32_ioctl(fd, DIO32_IOCTL_LED0, &val);

	if (ret)
	{
		errs++;
		printf("FAIL <---  (%d. failure reported, but shouldn't)\n", __LINE__);
	}

	// valid:
	val	= -1;
	ret	= dio32_ioctl(fd, DIO32_IOCTL_LED0, &val);

	if (ret)
	{
		errs++;
		printf("FAIL <---  (%d. failure reported, but shouldn't)\n", __LINE__);
	}

	// invalid: value
	val	= -2;
	ret	= dio32_ioctl(fd, DIO32_IOCTL_LED0, &val);

	if (ret == 0)
	{
		errs++;
		printf("FAIL <---  (%d. failure not reported: value)\n", __LINE__);
	}

	// invalid: value
	val	= 4;
	ret	= dio32_ioctl(fd, DIO32_IOCTL_LED0, &val);

	if (ret == 0)
	{
		errs++;
		printf("FAIL <---  (%d. failure not reported: value)\n", __LINE__);
	}

	errs	+= dio32_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	// Not testabe without additional equipment.
	// It can be verified via the "led" sample application.
	return(0);
}



/******************************************************************************
*
*	Function:	led3_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service DIO32_IOCTL_LED3.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int led3_test(int fd)
{
	int	errs	= 0;

	gsc_label("DIO32_IOCTL_LED3");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


