// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO32/DIO32_Linux_1.x_GSC_DN/samples/sbtest/lh_out_value.c $
// $Rev: 51807 $
// $Date: 2022-10-14 12:34:33 -0500 (Fri, 14 Oct 2022) $

// DIO32: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	int	errs	= 0;
	int	i;

	for (i = 0; i <= 31; i++)
		errs	+= input_response_service_test(fd, DIO32_IOCTL_LH_OUT_VALUE, i, lhovr_reg_list[i]);

	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	int	bit;
	u32	bit_mask;
	int	errs	= 0;
	u32	expect;
	int	port;
	u32	port_mask;
	u32	val;

	// For each Low-High Output Mask Register, verify that each mask bit takes
	// affect when the corresponding interrupt occurs.

	for (port = 0; port <= 31; port++)
	{
		port_mask	= 1L << port;

		for (bit = 0; bit <= 31; bit++)
		{
			if (port == bit)
				continue;

			bit_mask	= 1 << bit;

			// Prepare the device for the interrupt.
			errs	+= dio32_initialize			(fd, -1, 0);
			errs	+= dio32_deglitch_filter	(fd, -1, 0, 0, NULL);
			errs	+= dio32_deglitch_rate		(fd, -1, 0, 0, NULL);
			errs	+= dio32_io_out_clock		(fd, -1, 0, -1, DIO32_ACT_TX, 0x11111111, DIO32_IO_OUT_CLOCK_NONE, NULL);
			errs	+= dio32_io_out_dir			(fd, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, 0xFFFFFFFF, NULL);
			errs	+= dio32_io_out_tri_state	(fd, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, 0, NULL);
			errs	+= dio32_io_out_val			(fd, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, 0, NULL);

			// Configure the Low-High Output register.
			errs	+= dio32_lh_out_mask		(fd, -1, 0, port, DIO32_ACT_TX, 0xFFFFFFFF, bit_mask, NULL);
			errs	+= dio32_lh_out_value		(fd, -1, 0, port, DIO32_ACT_TX, 0xFFFFFFFF, 0xFFFFFFFF, NULL);
			errs	+= dio32_lh_out_tri_state	(fd, -1, 0, port, DIO32_ACT_TX, 0xFFFFFFFF, 0, NULL);

			// Generate the interrupt.
			errs	+= dio32_irq_lh_enable		(fd, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, port_mask, NULL);
			errs	+= dio32_io_out_val			(fd, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, port_mask, NULL);

			// Verify that the change took affect.
			errs	+= dio32_io_out_val			(fd, -1, 0, -1, DIO32_ACT_RX, 0xFFFFFFFF, 0, &val);
			expect	= port_mask | bit_mask;

			if (val != expect)
			{
				errs++;
				printf(	"FAIL <---  (%d. port %d, output failure: "
						"got 0x%08lX, expected 0x%08lX)\n",
						__LINE__,
						port,
						(long) val,
						(long) expect);
			}
		}
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	lh_out_value_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service DIO32_IOCTL_LH_OUT_VALUE.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int lh_out_value_test(int fd)
{
	int	errs	= 0;

	gsc_label("DIO32_IOCTL_LH_OUT_VALUE");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


