// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO32/DIO32_Linux_1.x_GSC_DN/utils/util_deglitch_rate.c $
// $Rev: 45663 $
// $Date: 2019-08-05 17:54:20 -0500 (Mon, 05 Aug 2019) $

// DIO32: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dio32_deglitch_rate
*
*	Purpose:
*
*		Provide a visual wrapper for the DIO32_IOCTL_DEGLITCH_RATE service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int dio32_deglitch_rate(int fd, int index, int verbose, s32 set, s32* get)
{
	int		errs;
	long	ns;
	int		ret;

	if (verbose)
		gsc_label_index("Deglitch Rate", index);

	ret		= dio32_ioctl(fd, DIO32_IOCTL_DEGLITCH_RATE, &set);
	errs	= ret ? 1 : 0;

	if (verbose)
	{
		printf("%s  (%ld * 100ns = ", errs ? "FAIL <---" : "PASS", (long) set);
		ns	= 100L * set;

		if ((ns % 1000000000) == 0)
			printf("%ld S", ns / 1000000000);
		else if ((ns % 1000000) == 0)
			printf("%ld ms", ns / 1000000);
		else if ((ns % 1000) == 0)
			printf("%ld us", ns / 1000);
		else
			printf("%ld ns", ns);

		printf(")\n");
	}

	if (get)
		get[0]	= set;

	return(errs);
}


