// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO32/DIO32_Linux_1.x_GSC_DN/utils/util_id.c $
// $Rev: 51801 $
// $Date: 2022-10-14 12:30:29 -0500 (Fri, 14 Oct 2022) $

// DIO32: Utilities: source file

#include "main.h"



//*****************************************************************************
static int _id_device_pci(int fd, int verbose)
{
	u32	didr;
	int	errs;
	u32	reg;

	if (verbose)
		gsc_label("Vendor ID");

	errs	= dio32_reg_read(fd, -1, 0, GSC_PCI_9056_VIDR, &reg);

	if (verbose)
		printf("0x%04lX      ", (long) reg);

	if (reg == 0x10B5)
	{
		if (verbose)
			printf("(PLX)\n");
	}
	else
	{
		errs++;

		if (verbose)
			printf("(UNKNOWN) FAIL <---\n");
	}

	if (verbose)
		gsc_label("Device ID");

	errs	+= dio32_reg_read(fd, -1, 0, GSC_PCI_9056_DIDR, &didr);

	if (verbose)
		printf("0x%04lX      ", (long) didr);

	if (didr == 0x9056)
	{
		if (verbose)
			printf("(PCI9056)\n");
	}
	else
	{
		errs++;

		if (verbose)
			printf("(UNKNOWN) FAIL <---\n");
	}

	if (verbose)
		gsc_label("Sub Vendor ID");

	errs	+= dio32_reg_read(fd, -1, 0, GSC_PCI_9056_SVID, &reg);

	if (verbose)
		printf("0x%04lX      ", (long) reg);

	if (reg == 0x10B5)
	{
		if (verbose)
			printf("(PLX)\n");
	}
	else
	{
		errs++;

		if (verbose)
			printf("(UNKNOWN) FAIL <---\n");
	}

	if (verbose)
		gsc_label("Subsystem ID");

	errs	+= dio32_reg_read(fd, -1, 0, GSC_PCI_9056_SID, &reg);

	if (verbose)
		printf("0x%04lX      ", (long) reg);

	if (didr == 0x9056)
	{
		if (reg == 0x2706)
		{
			if (verbose)
				printf("(DIO32)\n");
		}
		else
		{
			errs++;

			if (verbose)
				printf("(UNKNOWN) FAIL <---\n");
		}
	}
	else
	{
		errs++;

		if (verbose)
			printf("(UNKNOWN) FAIL <---\n");
	}

	return(errs);
}



//*****************************************************************************
static int _id_device_jumpers(int fd, int verbose)
{
	int		bit;
	int		errs	= 0;
	char	format[32];
	int		i;
	s32		qty;
	s32		sense;
	s32		value;

	if (verbose)
		gsc_label("User Jumpers");

	errs	+= dio32_query(fd, -1, 0, DIO32_QUERY_USER_JUMPER_QTY, &qty);
	errs	+= dio32_query(fd, -1, 0, DIO32_QUERY_USER_JUMPER_SENSE, &sense);
	errs	+= dio32_query(fd, -1, 0, DIO32_QUERY_USER_JUMPER_VALUE, &value);

	if (qty == 0)
	{
		if (verbose)
			printf("None\n");
	}
	else
	{
		sprintf(format, "0x%%0%ldlX  (", (long) ((qty + 3) / 4));

		if (verbose)
			printf(format, (long) value);

		for (i = qty - 1; i >= 0; i--)
		{
			bit	= 1 << i;
			bit	= bit & value;

			if (verbose == 0)
			{
			}
			else if (bit)
			{
				if (sense)
					printf("On");
				else
					printf("Off");
			}
			else
			{
				if (sense)
					printf("Off");
				else
					printf("On");
			}

			if ((i) && (verbose))
				printf(", ");
		}
	}

	if (verbose)
		printf(")\n");

	return(errs);
}



// ****************************************************************************
static int _show_prefix(int fd, int verbose)
{
	s32			bus;
	int			errs	= 0;
	s32			ff;
	const char*	tail;

	errs	+= dio32_query(fd, -1, 0, DIO32_QUERY_BUS_WIDTH, &bus);
	errs	+= dio32_query(fd, -1, 0, DIO32_QUERY_FORM_FACTOR, &ff);

	switch (bus)
	{
		default:

			errs++;

			if (verbose)
				printf("FAIL <---  (invalid bus size: %ld)\n", (long) bus);

		case 32:	tail	= "";	break;
		case 64:	tail	= "64";	break;
	}

	if ((errs == 0) && (verbose))
	{
		switch (ff)
		{
			default:
			case DIO32_QUERY_FF_UNKNOWN:								break;
			case DIO32_QUERY_FF_PCI:		printf("PCI%s-", tail);		break;
			case DIO32_QUERY_FF_PMC:		printf("PMC%s-", tail);		break;
			case DIO32_QUERY_FF_CPCI:		printf("CPCI%s-", tail);	break;
			case DIO32_QUERY_FF_PC104P:		printf("PC104P%s-", tail);	break;
			case DIO32_QUERY_FF_PCIE:		printf("PCIe-");			break;
		}
	}

	return(errs);
}



// ****************************************************************************
static int _show_transceivers(int fd, int verbose)
{
	int	errs;
	s32	ff;

	errs	= dio32_query(fd, -1, 0, DIO32_QUERY_XCVR_TYPE, &ff);

	switch (ff)
	{
		default:

			errs++;

			if (verbose)
				printf("FAIL <---  (invalid form factor: %ld)\n", (long) ff);

			break;

		case DIO32_QUERY_XCVR_UNKNOWN:

			if (verbose)
				printf("-\?\?\?\?");

			break;

		case DIO32_QUERY_XCVR_RS485:

			if (verbose)
				printf("-RS485");

			break;
	}

	return(errs);
}



// ****************************************************************************
static int _show_model(int fd, int verbose)
{
	int	errs;
	s32	type;

	errs	= dio32_query(fd, -1, 0, DIO32_QUERY_DEVICE_TYPE, &type);

	if (errs)
	{
	}
	else if (type != GSC_DEV_TYPE_DIO32)
	{
		errs++;

		if (verbose)
			printf("FAIL <---  (invalid device type: %ld)\n", (long) type);
	}
	else
	{
		errs	+= _show_prefix(fd, verbose);

		if (verbose)
			printf("DIO32A");

		errs	+= _show_transceivers(fd, verbose);
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	dio32_id_device
*
*	Purpose:
*
*		Identify the device.
*
*	Arguments:
*
*		fd		The handle to use to access the device.
*
*		index	The index of the device or -1 if we don't care.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int dio32_id_device(int fd, int index, int verbose)
{
	int	errs;

	if (verbose)
		gsc_label_index("Device", index);

	errs	= _show_model(fd, verbose);

	if (errs == 0)
		printf("\n");

	if (verbose)
		gsc_label_level_inc();

	errs	+= _id_device_pci(fd, verbose);
	errs	+= _id_device_jumpers(fd, verbose);

	if (verbose)
		gsc_label_level_dec();

	return(errs);
}



//*****************************************************************************
int dio32_id_driver(void)
{
	int	errs;

	errs	= os_id_driver(dio32_open, dio32_read, dio32_close);
	return(errs);
}


