// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO32/DIO32_Linux_1.x_GSC_DN/utils/util_io_out_clock.c $
// $Rev: 45663 $
// $Date: 2019-08-05 17:54:20 -0500 (Mon, 05 Aug 2019) $

// DIO32: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dio32_io_out_clock
*
*	Purpose:
*
*		Provide a visual wrapper for the DIO32_IOCTL_IO_OUT_CLOCK service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		port	The index of the port to access, or -1 to use mask.
*
*		action	DIO32_ACT_MX/RX/TX
*
*		mask	The mask of port bits to access.
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int dio32_io_out_clock(int fd, int index, int verbose, s32 port, s32 action, u32 mask, u32 set, u32* get)
{
	dio32_port_t	arg;
	char			buf[64];
	int				errs;
	char*			ptr;
	int				ret;

	if (verbose)
		gsc_label_index("Output Clock", index);

	arg.port	= port;
	arg.action	= action;
	arg.value	= set;
	arg.mask	= mask;

	ret		= dio32_ioctl(fd, DIO32_IOCTL_IO_OUT_CLOCK, &arg);
	errs	= ret ? 1 : 0;

	switch (arg.value)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case DIO32_IO_OUT_CLOCK_NONE:

			ptr	= "None, Discrete Output";
			break;

		case DIO32_IO_OUT_CLOCK_1:

			ptr	= "Clock 1";
			break;

		case DIO32_IO_OUT_CLOCK_2:

			ptr	= "Clock 2";
			break;

		case DIO32_IO_OUT_CLOCK_3:

			ptr	= "Clock 3";
			break;
	}

	if (verbose)
	{
		printf(	"%s  (mask 0x%08lX, %s)\n",
				errs ? "FAIL <---" : "PASS",
				(long) arg.mask,
				ptr);
	}

	if (get)
		get[0]	= arg.value;

	return(errs);
}


