// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO32/DIO32_Linux_1.x_GSC_DN/utils/util_led3.c $
// $Rev: 45663 $
// $Date: 2019-08-05 17:54:20 -0500 (Mon, 05 Aug 2019) $

// DIO32: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dio32_led3
*
*	Purpose:
*
*		Provide a visual wrapper for the DIO32_IOCTL_LED3 service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int dio32_led3(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("LED #3", index);

	ret		= dio32_ioctl(fd, DIO32_IOCTL_LED3, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case DIO32_LED_OFF:

			ptr	= "Off";
			break;

		case DIO32_LED_GREEN:

			ptr	= "Green";
			break;

		case DIO32_LED_RED:

			ptr	= "Red";
			break;

		case DIO32_LED_BOTH:

			ptr	= "Green + Red";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


