// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO32/DIO32_Linux_1.x_GSC_DN/utils/util_lh_out_tri_state.c $
// $Rev: 45663 $
// $Date: 2019-08-05 17:54:20 -0500 (Mon, 05 Aug 2019) $

// DIO32: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dio32_lh_out_tri_state
*
*	Purpose:
*
*		Provide a visual wrapper for the DIO32_IOCTL_LH_OUT_TRI_STATE service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		port	The index of the port to access.
*
*		action	DIO32_ACT_MX/RX/TX
*
*		mask	The mask of port bits to access.
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int dio32_lh_out_tri_state(int fd, int index, int verbose, s32 port, s32 action, u32 mask, u32 set, u32* get)
{
	dio32_port_t	arg;
	char			buf[64];
	int				errs;
	int				ret;

	if (verbose)
	{
		sprintf(buf, "Low->High Output Tri-State #%d", (int) port);
		gsc_label(buf);
	}

	arg.port	= port;
	arg.action	= action;
	arg.value	= set;
	arg.mask	= mask;

	ret		= dio32_ioctl(fd, DIO32_IOCTL_LH_OUT_TRI_STATE, &arg);
	errs	= ret ? 1 : 0;

	if (verbose)
	{
		printf(	"%s  (mask 0x%08lX, Tri-State 0x%08lX)\n",
				errs ? "FAIL <---" : "PASS",
				(long) arg.mask,
				(long) arg.value);
	}

	if (get)
		get[0]	= arg.value;

	return(errs);
}


