// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO32/DIO32_Linux_1.x_GSC_DN/utils/util_query.c $
// $Rev: 54527 $
// $Date: 2024-05-28 09:24:59 -0500 (Tue, 28 May 2024) $

// DIO32: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dio32_query
*
*	Purpose:
*
*		Provide a visual wrapper for the DIO32_IOCTL_QUERY service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely? 0 = no, !0 = yes
*
*		set		This is the query option to access.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int dio32_query(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs	= 0;
	const char*	ptr;
	s32			query	= set;
	int			ret;

	switch (query)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Query Error (#%ld)", (long) query);
			break;

		case DIO32_QUERY_BOARD_INDEX:		ptr	= "Board Index";	break;
		case DIO32_QUERY_BUS_WIDTH:			ptr	= "Bus Width";		break;
		case DIO32_QUERY_COUNT:				ptr	= "Query Options";	break;
		case DIO32_QUERY_DEVICE_TYPE:		ptr	= "Device Type";	break;
		case DIO32_QUERY_FORM_FACTOR:		ptr	= "Form Factor";	break;
		case DIO32_QUERY_IO_BITS:			ptr	= "I/O Bits";		break;
		case DIO32_QUERY_USER_JUMPER_QTY:	ptr	= "Jumper Qty";		break;
		case DIO32_QUERY_USER_JUMPER_SENSE:	ptr	= "Jumper Sense";	break;
		case DIO32_QUERY_USER_JUMPER_VALUE:	ptr	= "Jumper Value";	break;
		case DIO32_QUERY_XCVR_TYPE:			ptr	= "Xcvr Type";		break;
	}

	if (verbose)
		gsc_label_index(ptr, index);

	ret		= dio32_ioctl(fd, DIO32_IOCTL_QUERY, &set);
	errs	+= ret ? 1 : 0;

	switch (query)
	{
		default:

			errs++;
			gsc_label_long_comma_buf(set, buf);
			break;

		case DIO32_QUERY_BOARD_INDEX:
		case DIO32_QUERY_COUNT:
		case DIO32_QUERY_USER_JUMPER_QTY:

			gsc_label_long_comma_buf(set, buf);
			break;

		case DIO32_QUERY_USER_JUMPER_VALUE:

			sprintf(buf, "0x%lX", (long) set);
			break;

		case DIO32_QUERY_BUS_WIDTH:
		case DIO32_QUERY_IO_BITS:

			sprintf(buf, "%ld Bits", (long) set);
			break;

		case DIO32_QUERY_USER_JUMPER_SENSE:

			sprintf(buf, "%s == On", set ? "1" : "0");
			break;

		case DIO32_QUERY_DEVICE_TYPE:

			if (set == GSC_DEV_TYPE_DIO32)
			{
				strcpy(buf, "DIO32");
			}
			else
			{
				errs++;
				sprintf(buf, "INVALID: %ld", (long) set);
			}

			break;

		case DIO32_QUERY_FORM_FACTOR:

			switch (set)
			{
				default:

					errs++;
					sprintf(buf, "Invalid option: %ld", (long) set);
					break;

				case DIO32_QUERY_FF_UNKNOWN:	strcpy(buf, "unknown");		break;
				case DIO32_QUERY_FF_PCI:		strcpy(buf, "PCI");			break;
				case DIO32_QUERY_FF_PMC:		strcpy(buf, "PMC");			break;
				case DIO32_QUERY_FF_CPCI:		strcpy(buf, "CPCI");		break;
				case DIO32_QUERY_FF_PC104P:		strcpy(buf, "PC/104+");		break;
				case DIO32_QUERY_FF_PCIE:		strcpy(buf, "PCI Express");	break;
			}

			break;

		case DIO32_QUERY_XCVR_TYPE:

			switch (set)
			{
				default:

					errs++;
					sprintf(buf, "Invalid option: %ld", (long) set);
					break;

				case DIO32_QUERY_XCVR_UNKNOWN:	strcpy(buf, "unknown");	break;
				case DIO32_QUERY_XCVR_RS485:	strcpy(buf, "RS485");	break;
			}

			break;
	}

	if (verbose)
	{
		if (errs)
			printf("FAIL <---  (%s)\n", buf);
		else
			printf("%s\n", buf);
	}

	if (get)
		get[0]	= set;

	return(errs);
}


