// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO32/DIO32_Linux_1.x_GSC_DN/utils/util_reg.c $
// $Rev: 54527 $
// $Date: 2024-05-28 09:24:59 -0500 (Tue, 28 May 2024) $

// DIO32: Utilities: source file

#include "main.h"



// macros *********************************************************************

#define	_GSC_REG(a)							"GSC " #a, DIO32_GSC_##a, 0, 0

#define	SHOW_FIELD(beg,end,eol,opts,desc)	gsc_reg_field_show(width + 10,24,value,(beg),(end),(eol),(opts),(desc))




//*****************************************************************************
static int _frr_detail(int fd, int supported, u32 value, int width)
{
	static const char* const	form[]	=
	{
		"RESERVED",
		"PCI",
		"PMC",
		"cPCI",
		"PC104+",
		"PCI Express",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED",
		"RESERVED"
	};

	static const char* const	fw_size[]	= { "32-bit",	"64-bit"	};

	long	v;

	SHOW_FIELD(31, 31, 1, NULL,		"Reserved"		);
	SHOW_FIELD(30, 30, 1, NULL,		"Reserved"		);
	SHOW_FIELD(29, 29, 1, NULL,		"Reserved"		);
	SHOW_FIELD(28, 28, 1, fw_size,	"Firmware Size"	);
	SHOW_FIELD(27, 24, 1, form,		"Form Factor"	);
	SHOW_FIELD(23, 16, 0, NULL,		"PCB Revision"	);
	v	= (value & 0xFF0000) >> 16;

	if (v == 0)
		printf("No Revision\n");
	else
		printf("Rev. %c\n", (char) ('A' + v - 1));

	SHOW_FIELD(15,  8, 1, NULL,	"Firmware Type"		);
	SHOW_FIELD( 7,  0, 0, NULL,	"Firmware Revision"	);
	v	= value & 0xFF;
	printf("%02lX\n", v);
	return(0);
}



//*****************************************************************************
static int _fr_detail(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(31, 0, 1, NULL,	"Reserved"	);
	return(0);
}



//*****************************************************************************
static int _bcr_detail(int fd, int supported, u32 value, int width)
{
	static const char* const	disabled[]	= { "Disabled",	"Enabled"	};
	static const char* const	idle[]		= { "Idle",		"Active"	};
	static const char* const	led[]		= { "Off", "Green", "Red", "Green + Red" };

	SHOW_FIELD(31, 31, 1, idle,		"Board Reset"	);
	SHOW_FIELD(30, 30, 1, disabled,	"Test Mode"		);
	SHOW_FIELD(29, 29, 1, disabled,	"Test Mode"		);
	SHOW_FIELD(28, 12, 1, NULL,		"Reserved"		);
	SHOW_FIELD(11, 10, 1, led,		"LED D5"		);
	SHOW_FIELD( 9,  8, 1, led,		"LED D4"		);
	SHOW_FIELD( 7,  6, 1, led,		"LED D3"		);
	SHOW_FIELD( 5,  4, 1, led,		"LED D2"		);
	SHOW_FIELD( 3,  2, 1, led,		"LED D1"		);
	SHOW_FIELD( 1,  0, 1, led,		"LED D0"		);
	return(0);
}



//*****************************************************************************
static int _bsr_detail(int fd, int supported, u32 value, int width)
{
	static const char* const	on[]	= { "On",	"Off"		};
	static const char* const	idle[]	= { "Idle",	"Active"	};

	SHOW_FIELD(31, 31, 1, idle,		"Board Reset"			);
	SHOW_FIELD(30, 16, 1, NULL,		"Number of I/O Bits"	);
	SHOW_FIELD(15, 14, 1, NULL,		"Reserved"				);
	SHOW_FIELD(13,  4, 1, NULL,		"Configuration ID"		);
	SHOW_FIELD( 3,  3, 1, on,		"User Jumper ID4"		);
	SHOW_FIELD( 2,  2, 1, on,		"User Jumper ID3"		);
	SHOW_FIELD( 1,  1, 1, on,		"User Jumper ID2"		);
	SHOW_FIELD( 0,  0, 1, on,		"User Jumper ID1"		);
	return(0);
}



//*****************************************************************************
static int _itr_detail(int fd, int supported, u32 value, int width)
{
	static const char* const	on[]	= { "On",	"Off"	};

	SHOW_FIELD(31,  2, 1, NULL,		"Reserved"			);
	SHOW_FIELD( 1,  1, 1, on,		"Hi-Lo Interrupt"	);
	SHOW_FIELD( 0,  0, 1, on,		"Lo-Hi Interrupd"	);
	return(0);
}



//*****************************************************************************
static int _ocr_detail(int fd, int supported, u32 value, int width)
{
	static const char* const	clock[]	= { "None", "Clock 1", "Clock 2", "Clock 3" };

	SHOW_FIELD(31, 30, 1, NULL,		"Reserved"	);
	SHOW_FIELD(29, 28, 1, clock,	"D28 Clock"	);
	SHOW_FIELD(27, 26, 1, NULL,		"Reserved"	);
	SHOW_FIELD(25, 24, 1, clock,	"D24 Clock"	);
	SHOW_FIELD(23, 22, 1, NULL,		"Reserved"	);
	SHOW_FIELD(21, 20, 1, clock,	"D20 Clock"	);
	SHOW_FIELD(19, 18, 1, NULL,		"Reserved"	);
	SHOW_FIELD(17, 16, 1, clock,	"D16 Clock"	);
	SHOW_FIELD(15, 14, 1, NULL,		"Reserved"	);
	SHOW_FIELD(13, 12, 1, clock,	"D12 Clock"	);
	SHOW_FIELD(11, 10, 1, NULL,		"Reserved"	);
	SHOW_FIELD( 9,  8, 1, clock,	"D8 Clock"	);
	SHOW_FIELD( 7,  6, 1, NULL,		"Reserved"	);
	SHOW_FIELD( 5,  4, 1, clock,	"D4 Clock"	);
	SHOW_FIELD( 3,  2, 1, NULL,		"Reserved"	);
	SHOW_FIELD( 1,  0, 1, clock,	"D0 Clock"	);
	return(0);
}



//*****************************************************************************
static int _dgcr_detail(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(31,  8, 1, NULL,	"Sample Rate"	);
	SHOW_FIELD( 7,  0, 1, NULL,	"Glitch Filter"	);
	return(0);
}



// variables ******************************************************************

static gsc_reg_def_t	_gsc[]	=
{
	{ _GSC_REG(FRR),		0,	_frr_detail,	"Firmware Revision Register"				},
	{ _GSC_REG(FR),			0,	_fr_detail,		"Features Register"							},
	{ _GSC_REG(BCR),		0,	_bcr_detail,	"Board Control Register"					},
	{ _GSC_REG(BSR),		0,	_bsr_detail,	"Board Status Register"						},
	{ _GSC_REG(ITR),		0,	_itr_detail,	"Interrupt Type Register"					},
	{ _GSC_REG(IODR),		0,	NULL,			"I/O Direction Register (0-in,1=out)"		},
	{ _GSC_REG(PIR),		0,	NULL,			"Pin Input Register"						},
	{ _GSC_REG(DGIR),		0,	NULL,			"Deglitched Input Register"					},
	{ _GSC_REG(OTSR),		0,	NULL,			"Output Tri-State Register"					},
	{ _GSC_REG(OVR),		0,	NULL,			"Output Value Register"						},
	{ _GSC_REG(OCR),		0,	_ocr_detail,	"Output Clock Register"						},
	{ _GSC_REG(DGCR),		0,	_dgcr_detail,	"Deglitch Control Register"					},
	{ _GSC_REG(PC1R),		0,	NULL,			"Programmable Clock 1 Register"				},
	{ _GSC_REG(PC1PWMDR),	0,	NULL,			"Programmable Clock 1 PWM Divider Register"	},
	{ _GSC_REG(PC1FR),		0,	NULL,			"Programmable Clock 1 Frequency Register"	},
	{ _GSC_REG(PC2R),		0,	NULL,			"Programmable Clock 2 Register"				},
	{ _GSC_REG(PC2PWMDR),	0,	NULL,			"Programmable Clock 2 PWM Divider Register"	},
	{ _GSC_REG(PC2FR),		0,	NULL,			"Programmable Clock 2 Frequency Register"	},
	{ _GSC_REG(PC3R),		0,	NULL,			"Programmable Clock 3 Register"				},
	{ _GSC_REG(PC3PWMDR),	0,	NULL,			"Programmable Clock 3 PWM Divider Register"	},
	{ _GSC_REG(PC3FR),		0,	NULL,			"Programmable Clock 3 Frequency Register"	},
	{ _GSC_REG(LHIER),		0,	NULL,			"Low-to-High Interrupt Enable Register"			},
	{ _GSC_REG(LHISCR),		0,	NULL,			"Low-to-High Interrupt Status/Clear Register"	},
	{ _GSC_REG(HLIER),		0,	NULL,			"High-to-Low Interrupt Enable Register"			},
	{ _GSC_REG(HLISCR),		0,	NULL,			"High-to-Low Interrupt Status/Clear Register"	},

	{ _GSC_REG(D00LHOMR),	0,	NULL,			"D0 Low-to-High Output Mask Register"		},
	{ _GSC_REG(D00LHOTSR),	0,	NULL,			"D0 Low-to-High Output Tri-State Register"	},
	{ _GSC_REG(D00LHOVR),	0,	NULL,			"D0 Low-to-High Output Value Register"		},
	{ _GSC_REG(D00HLOMR),	0,	NULL,			"D0 High-to-Low Output Mask Register"		},
	{ _GSC_REG(D00HLOTSR),	0,	NULL,			"D0 High-to-Low Output Tri-State Register"	},
	{ _GSC_REG(D00HLOVR),	0,	NULL,			"D0 High-to-Low Output Value Register"		},

	{ _GSC_REG(D01LHOMR),	0,	NULL,			"D1 Low-to-High Output Mask Register"		},
	{ _GSC_REG(D01LHOTSR),	0,	NULL,			"D1 Low-to-High Output Tri-State Register"	},
	{ _GSC_REG(D01LHOVR),	0,	NULL,			"D1 Low-to-High Output Value Register"		},
	{ _GSC_REG(D01HLOMR),	0,	NULL,			"D1 High-to-Low Output Mask Register"		},
	{ _GSC_REG(D01HLOTSR),	0,	NULL,			"D1 High-to-Low Output Tri-State Register"	},
	{ _GSC_REG(D01HLOVR),	0,	NULL,			"D1 High-to-Low Output Value Register"		},

	{ _GSC_REG(D02LHOMR),	0,	NULL,			"D2 Low-to-High Output Mask Register"		},
	{ _GSC_REG(D02LHOTSR),	0,	NULL,			"D2 Low-to-High Output Tri-State Register"	},
	{ _GSC_REG(D02LHOVR),	0,	NULL,			"D2 Low-to-High Output Value Register"		},
	{ _GSC_REG(D02HLOMR),	0,	NULL,			"D2 High-to-Low Output Mask Register"		},
	{ _GSC_REG(D02HLOTSR),	0,	NULL,			"D2 High-to-Low Output Tri-State Register"	},
	{ _GSC_REG(D02HLOVR),	0,	NULL,			"D2 High-to-Low Output Value Register"		},

	{ _GSC_REG(D03LHOMR),	0,	NULL,			"D3 Low-to-High Output Mask Register"		},
	{ _GSC_REG(D03LHOTSR),	0,	NULL,			"D3 Low-to-High Output Tri-State Register"	},
	{ _GSC_REG(D03LHOVR),	0,	NULL,			"D3 Low-to-High Output Value Register"		},
	{ _GSC_REG(D03HLOMR),	0,	NULL,			"D3 High-to-Low Output Mask Register"		},
	{ _GSC_REG(D03HLOTSR),	0,	NULL,			"D3 High-to-Low Output Tri-State Register"	},
	{ _GSC_REG(D03HLOVR),	0,	NULL,			"D3 High-to-Low Output Value Register"		},

	{ _GSC_REG(D04LHOMR),	0,	NULL,			"D4 Low-to-High Output Mask Register"		},
	{ _GSC_REG(D04LHOTSR),	0,	NULL,			"D4 Low-to-High Output Tri-State Register"	},
	{ _GSC_REG(D04LHOVR),	0,	NULL,			"D4 Low-to-High Output Value Register"		},
	{ _GSC_REG(D04HLOMR),	0,	NULL,			"D4 High-to-Low Output Mask Register"		},
	{ _GSC_REG(D04HLOTSR),	0,	NULL,			"D4 High-to-Low Output Tri-State Register"	},
	{ _GSC_REG(D04HLOVR),	0,	NULL,			"D4 High-to-Low Output Value Register"		},

	{ _GSC_REG(D05LHOMR),	0,	NULL,			"D5 Low-to-High Output Mask Register"		},
	{ _GSC_REG(D05LHOTSR),	0,	NULL,			"D5 Low-to-High Output Tri-State Register"	},
	{ _GSC_REG(D05LHOVR),	0,	NULL,			"D5 Low-to-High Output Value Register"		},
	{ _GSC_REG(D05HLOMR),	0,	NULL,			"D5 High-to-Low Output Mask Register"		},
	{ _GSC_REG(D05HLOTSR),	0,	NULL,			"D5 High-to-Low Output Tri-State Register"	},
	{ _GSC_REG(D05HLOVR),	0,	NULL,			"D5 High-to-Low Output Value Register"		},

	{ _GSC_REG(D06LHOMR),	0,	NULL,			"D6 Low-to-High Output Mask Register"		},
	{ _GSC_REG(D06LHOTSR),	0,	NULL,			"D6 Low-to-High Output Tri-State Register"	},
	{ _GSC_REG(D06LHOVR),	0,	NULL,			"D6 Low-to-High Output Value Register"		},
	{ _GSC_REG(D06HLOMR),	0,	NULL,			"D6 High-to-Low Output Mask Register"		},
	{ _GSC_REG(D06HLOTSR),	0,	NULL,			"D6 High-to-Low Output Tri-State Register"	},
	{ _GSC_REG(D06HLOVR),	0,	NULL,			"D6 High-to-Low Output Value Register"		},

	{ _GSC_REG(D07LHOMR),	0,	NULL,			"D7 Low-to-High Output Mask Register"		},
	{ _GSC_REG(D07LHOTSR),	0,	NULL,			"D7 Low-to-High Output Tri-State Register"	},
	{ _GSC_REG(D07LHOVR),	0,	NULL,			"D7 Low-to-High Output Value Register"		},
	{ _GSC_REG(D07HLOMR),	0,	NULL,			"D7 High-to-Low Output Mask Register"		},
	{ _GSC_REG(D07HLOTSR),	0,	NULL,			"D7 High-to-Low Output Tri-State Register"	},
	{ _GSC_REG(D07HLOVR),	0,	NULL,			"D7 High-to-Low Output Value Register"		},

	{ _GSC_REG(D08LHOMR),	0,	NULL,			"D8 Low-to-High Output Mask Register"		},
	{ _GSC_REG(D08LHOTSR),	0,	NULL,			"D8 Low-to-High Output Tri-State Register"	},
	{ _GSC_REG(D08LHOVR),	0,	NULL,			"D8 Low-to-High Output Value Register"		},
	{ _GSC_REG(D08HLOMR),	0,	NULL,			"D8 High-to-Low Output Mask Register"		},
	{ _GSC_REG(D08HLOTSR),	0,	NULL,			"D8 High-to-Low Output Tri-State Register"	},
	{ _GSC_REG(D08HLOVR),	0,	NULL,			"D8 High-to-Low Output Value Register"		},

	{ _GSC_REG(D09LHOMR),	0,	NULL,			"D9 Low-to-High Output Mask Register"		},
	{ _GSC_REG(D09LHOTSR),	0,	NULL,			"D9 Low-to-High Output Tri-State Register"	},
	{ _GSC_REG(D09LHOVR),	0,	NULL,			"D9 Low-to-High Output Value Register"		},
	{ _GSC_REG(D09HLOMR),	0,	NULL,			"D9 High-to-Low Output Mask Register"		},
	{ _GSC_REG(D09HLOTSR),	0,	NULL,			"D9 High-to-Low Output Tri-State Register"	},
	{ _GSC_REG(D09HLOVR),	0,	NULL,			"D9 High-to-Low Output Value Register"		},

	{ _GSC_REG(D10LHOMR),	0,	NULL,			"D10 Low-to-High Output Mask Register"		},
	{ _GSC_REG(D10LHOTSR),	0,	NULL,			"D10 Low-to-High Output Tri-State Register"	},
	{ _GSC_REG(D10LHOVR),	0,	NULL,			"D10 Low-to-High Output Value Register"		},
	{ _GSC_REG(D10HLOMR),	0,	NULL,			"D10 High-to-Low Output Mask Register"		},
	{ _GSC_REG(D10HLOTSR),	0,	NULL,			"D10 High-to-Low Output Tri-State Register"	},
	{ _GSC_REG(D10HLOVR),	0,	NULL,			"D10 High-to-Low Output Value Register"		},

	{ _GSC_REG(D11LHOMR),	0,	NULL,			"D11 Low-to-High Output Mask Register"		},
	{ _GSC_REG(D11LHOTSR),	0,	NULL,			"D11 Low-to-High Output Tri-State Register"	},
	{ _GSC_REG(D11LHOVR),	0,	NULL,			"D11 Low-to-High Output Value Register"		},
	{ _GSC_REG(D11HLOMR),	0,	NULL,			"D11 High-to-Low Output Mask Register"		},
	{ _GSC_REG(D11HLOTSR),	0,	NULL,			"D11 High-to-Low Output Tri-State Register"	},
	{ _GSC_REG(D11HLOVR),	0,	NULL,			"D11 High-to-Low Output Value Register"		},

	{ _GSC_REG(D12LHOMR),	0,	NULL,			"D12 Low-to-High Output Mask Register"		},
	{ _GSC_REG(D12LHOTSR),	0,	NULL,			"D12 Low-to-High Output Tri-State Register"	},
	{ _GSC_REG(D12LHOVR),	0,	NULL,			"D12 Low-to-High Output Value Register"		},
	{ _GSC_REG(D12HLOMR),	0,	NULL,			"D12 High-to-Low Output Mask Register"		},
	{ _GSC_REG(D12HLOTSR),	0,	NULL,			"D12 High-to-Low Output Tri-State Register"	},
	{ _GSC_REG(D12HLOVR),	0,	NULL,			"D12 High-to-Low Output Value Register"		},

	{ _GSC_REG(D13LHOMR),	0,	NULL,			"D13 Low-to-High Output Mask Register"		},
	{ _GSC_REG(D13LHOTSR),	0,	NULL,			"D13 Low-to-High Output Tri-State Register"	},
	{ _GSC_REG(D13LHOVR),	0,	NULL,			"D13 Low-to-High Output Value Register"		},
	{ _GSC_REG(D13HLOMR),	0,	NULL,			"D13 High-to-Low Output Mask Register"		},
	{ _GSC_REG(D13HLOTSR),	0,	NULL,			"D13 High-to-Low Output Tri-State Register"	},
	{ _GSC_REG(D13HLOVR),	0,	NULL,			"D13 High-to-Low Output Value Register"		},

	{ _GSC_REG(D14LHOMR),	0,	NULL,			"D14 Low-to-High Output Mask Register"		},
	{ _GSC_REG(D14LHOTSR),	0,	NULL,			"D14 Low-to-High Output Tri-State Register"	},
	{ _GSC_REG(D14LHOVR),	0,	NULL,			"D14 Low-to-High Output Value Register"		},
	{ _GSC_REG(D14HLOMR),	0,	NULL,			"D14 High-to-Low Output Mask Register"		},
	{ _GSC_REG(D14HLOTSR),	0,	NULL,			"D14 High-to-Low Output Tri-State Register"	},
	{ _GSC_REG(D14HLOVR),	0,	NULL,			"D14 High-to-Low Output Value Register"		},

	{ _GSC_REG(D15LHOMR),	0,	NULL,			"D15 Low-to-High Output Mask Register"		},
	{ _GSC_REG(D15LHOTSR),	0,	NULL,			"D15 Low-to-High Output Tri-State Register"	},
	{ _GSC_REG(D15LHOVR),	0,	NULL,			"D15 Low-to-High Output Value Register"		},
	{ _GSC_REG(D15HLOMR),	0,	NULL,			"D15 High-to-Low Output Mask Register"		},
	{ _GSC_REG(D15HLOTSR),	0,	NULL,			"D15 High-to-Low Output Tri-State Register"	},
	{ _GSC_REG(D15HLOVR),	0,	NULL,			"D15 High-to-Low Output Value Register"		},

	{ _GSC_REG(D16LHOMR),	0,	NULL,			"D16 Low-to-High Output Mask Register"		},
	{ _GSC_REG(D16LHOTSR),	0,	NULL,			"D16 Low-to-High Output Tri-State Register"	},
	{ _GSC_REG(D16LHOVR),	0,	NULL,			"D16 Low-to-High Output Value Register"		},
	{ _GSC_REG(D16HLOMR),	0,	NULL,			"D16 High-to-Low Output Mask Register"		},
	{ _GSC_REG(D16HLOTSR),	0,	NULL,			"D16 High-to-Low Output Tri-State Register"	},
	{ _GSC_REG(D16HLOVR),	0,	NULL,			"D16 High-to-Low Output Value Register"		},

	{ _GSC_REG(D17LHOMR),	0,	NULL,			"D17 Low-to-High Output Mask Register"		},
	{ _GSC_REG(D17LHOTSR),	0,	NULL,			"D17 Low-to-High Output Tri-State Register"	},
	{ _GSC_REG(D17LHOVR),	0,	NULL,			"D17 Low-to-High Output Value Register"		},
	{ _GSC_REG(D17HLOMR),	0,	NULL,			"D17 High-to-Low Output Mask Register"		},
	{ _GSC_REG(D17HLOTSR),	0,	NULL,			"D17 High-to-Low Output Tri-State Register"	},
	{ _GSC_REG(D17HLOVR),	0,	NULL,			"D17 High-to-Low Output Value Register"		},

	{ _GSC_REG(D18LHOMR),	0,	NULL,			"D18 Low-to-High Output Mask Register"		},
	{ _GSC_REG(D18LHOTSR),	0,	NULL,			"D18 Low-to-High Output Tri-State Register"	},
	{ _GSC_REG(D18LHOVR),	0,	NULL,			"D18 Low-to-High Output Value Register"		},
	{ _GSC_REG(D18HLOMR),	0,	NULL,			"D18 High-to-Low Output Mask Register"		},
	{ _GSC_REG(D18HLOTSR),	0,	NULL,			"D18 High-to-Low Output Tri-State Register"	},
	{ _GSC_REG(D18HLOVR),	0,	NULL,			"D18 High-to-Low Output Value Register"		},

	{ _GSC_REG(D19LHOMR),	0,	NULL,			"D19 Low-to-High Output Mask Register"		},
	{ _GSC_REG(D19LHOTSR),	0,	NULL,			"D19 Low-to-High Output Tri-State Register"	},
	{ _GSC_REG(D19LHOVR),	0,	NULL,			"D19 Low-to-High Output Value Register"		},
	{ _GSC_REG(D19HLOMR),	0,	NULL,			"D19 High-to-Low Output Mask Register"		},
	{ _GSC_REG(D19HLOTSR),	0,	NULL,			"D19 High-to-Low Output Tri-State Register"	},
	{ _GSC_REG(D19HLOVR),	0,	NULL,			"D19 High-to-Low Output Value Register"		},

	{ _GSC_REG(D20LHOMR),	0,	NULL,			"D20 Low-to-High Output Mask Register"		},
	{ _GSC_REG(D20LHOTSR),	0,	NULL,			"D20 Low-to-High Output Tri-State Register"	},
	{ _GSC_REG(D20LHOVR),	0,	NULL,			"D20 Low-to-High Output Value Register"		},
	{ _GSC_REG(D20HLOMR),	0,	NULL,			"D20 High-to-Low Output Mask Register"		},
	{ _GSC_REG(D20HLOTSR),	0,	NULL,			"D20 High-to-Low Output Tri-State Register"	},
	{ _GSC_REG(D20HLOVR),	0,	NULL,			"D20 High-to-Low Output Value Register"		},

	{ _GSC_REG(D21LHOMR),	0,	NULL,			"D21 Low-to-High Output Mask Register"		},
	{ _GSC_REG(D21LHOTSR),	0,	NULL,			"D21 Low-to-High Output Tri-State Register"	},
	{ _GSC_REG(D21LHOVR),	0,	NULL,			"D21 Low-to-High Output Value Register"		},
	{ _GSC_REG(D21HLOMR),	0,	NULL,			"D21 High-to-Low Output Mask Register"		},
	{ _GSC_REG(D21HLOTSR),	0,	NULL,			"D21 High-to-Low Output Tri-State Register"	},
	{ _GSC_REG(D21HLOVR),	0,	NULL,			"D21 High-to-Low Output Value Register"		},

	{ _GSC_REG(D22LHOMR),	0,	NULL,			"D22 Low-to-High Output Mask Register"		},
	{ _GSC_REG(D22LHOTSR),	0,	NULL,			"D22 Low-to-High Output Tri-State Register"	},
	{ _GSC_REG(D22LHOVR),	0,	NULL,			"D22 Low-to-High Output Value Register"		},
	{ _GSC_REG(D22HLOMR),	0,	NULL,			"D22 High-to-Low Output Mask Register"		},
	{ _GSC_REG(D22HLOTSR),	0,	NULL,			"D22 High-to-Low Output Tri-State Register"	},
	{ _GSC_REG(D22HLOVR),	0,	NULL,			"D22 High-to-Low Output Value Register"		},

	{ _GSC_REG(D23LHOMR),	0,	NULL,			"D23 Low-to-High Output Mask Register"		},
	{ _GSC_REG(D23LHOTSR),	0,	NULL,			"D23 Low-to-High Output Tri-State Register"	},
	{ _GSC_REG(D23LHOVR),	0,	NULL,			"D23 Low-to-High Output Value Register"		},
	{ _GSC_REG(D23HLOMR),	0,	NULL,			"D23 High-to-Low Output Mask Register"		},
	{ _GSC_REG(D23HLOTSR),	0,	NULL,			"D23 High-to-Low Output Tri-State Register"	},
	{ _GSC_REG(D23HLOVR),	0,	NULL,			"D23 High-to-Low Output Value Register"		},

	{ _GSC_REG(D24LHOMR),	0,	NULL,			"D24 Low-to-High Output Mask Register"		},
	{ _GSC_REG(D24LHOTSR),	0,	NULL,			"D24 Low-to-High Output Tri-State Register"	},
	{ _GSC_REG(D24LHOVR),	0,	NULL,			"D24 Low-to-High Output Value Register"		},
	{ _GSC_REG(D24HLOMR),	0,	NULL,			"D24 High-to-Low Output Mask Register"		},
	{ _GSC_REG(D24HLOTSR),	0,	NULL,			"D24 High-to-Low Output Tri-State Register"	},
	{ _GSC_REG(D24HLOVR),	0,	NULL,			"D24 High-to-Low Output Value Register"		},

	{ _GSC_REG(D25LHOMR),	0,	NULL,			"D25 Low-to-High Output Mask Register"		},
	{ _GSC_REG(D25LHOTSR),	0,	NULL,			"D25 Low-to-High Output Tri-State Register"	},
	{ _GSC_REG(D25LHOVR),	0,	NULL,			"D25 Low-to-High Output Value Register"		},
	{ _GSC_REG(D25HLOMR),	0,	NULL,			"D25 High-to-Low Output Mask Register"		},
	{ _GSC_REG(D25HLOTSR),	0,	NULL,			"D25 High-to-Low Output Tri-State Register"	},
	{ _GSC_REG(D25HLOVR),	0,	NULL,			"D25 High-to-Low Output Value Register"		},

	{ _GSC_REG(D26LHOMR),	0,	NULL,			"D26 Low-to-High Output Mask Register"		},
	{ _GSC_REG(D26LHOTSR),	0,	NULL,			"D26 Low-to-High Output Tri-State Register"	},
	{ _GSC_REG(D26LHOVR),	0,	NULL,			"D26 Low-to-High Output Value Register"		},
	{ _GSC_REG(D26HLOMR),	0,	NULL,			"D26 High-to-Low Output Mask Register"		},
	{ _GSC_REG(D26HLOTSR),	0,	NULL,			"D26 High-to-Low Output Tri-State Register"	},
	{ _GSC_REG(D26HLOVR),	0,	NULL,			"D26 High-to-Low Output Value Register"		},

	{ _GSC_REG(D27LHOMR),	0,	NULL,			"D27 Low-to-High Output Mask Register"		},
	{ _GSC_REG(D27LHOTSR),	0,	NULL,			"D27 Low-to-High Output Tri-State Register"	},
	{ _GSC_REG(D27LHOVR),	0,	NULL,			"D27 Low-to-High Output Value Register"		},
	{ _GSC_REG(D27HLOMR),	0,	NULL,			"D27 High-to-Low Output Mask Register"		},
	{ _GSC_REG(D27HLOTSR),	0,	NULL,			"D27 High-to-Low Output Tri-State Register"	},
	{ _GSC_REG(D27HLOVR),	0,	NULL,			"D27 High-to-Low Output Value Register"		},

	{ _GSC_REG(D28LHOMR),	0,	NULL,			"D28 Low-to-High Output Mask Register"		},
	{ _GSC_REG(D28LHOTSR),	0,	NULL,			"D28 Low-to-High Output Tri-State Register"	},
	{ _GSC_REG(D28LHOVR),	0,	NULL,			"D28 Low-to-High Output Value Register"		},
	{ _GSC_REG(D28HLOMR),	0,	NULL,			"D28 High-to-Low Output Mask Register"		},
	{ _GSC_REG(D28HLOTSR),	0,	NULL,			"D28 High-to-Low Output Tri-State Register"	},
	{ _GSC_REG(D28HLOVR),	0,	NULL,			"D28 High-to-Low Output Value Register"		},

	{ _GSC_REG(D29LHOMR),	0,	NULL,			"D29 Low-to-High Output Mask Register"		},
	{ _GSC_REG(D29LHOVR),	0,	NULL,			"D29 Low-to-High Output Value Register"		},
	{ _GSC_REG(D29LHOTSR),	0,	NULL,			"D29 Low-to-High Output Tri-State Register"	},
	{ _GSC_REG(D29HLOMR),	0,	NULL,			"D29 High-to-Low Output Mask Register"		},
	{ _GSC_REG(D29HLOVR),	0,	NULL,			"D29 High-to-Low Output Value Register"		},
	{ _GSC_REG(D29HLOTSR),	0,	NULL,			"D29 High-to-Low Output Tri-State Register"	},

	{ _GSC_REG(D30LHOMR),	0,	NULL,			"D30 Low-to-High Output Mask Register"		},
	{ _GSC_REG(D30LHOTSR),	0,	NULL,			"D30 Low-to-High Output Tri-State Register"	},
	{ _GSC_REG(D30LHOVR),	0,	NULL,			"D30 Low-to-High Output Value Register"		},
	{ _GSC_REG(D30HLOMR),	0,	NULL,			"D30 High-to-Low Output Mask Register"		},
	{ _GSC_REG(D30HLOTSR),	0,	NULL,			"D30 High-to-Low Output Tri-State Register"	},
	{ _GSC_REG(D30HLOVR),	0,	NULL,			"D30 High-to-Low Output Value Register"		},

	{ _GSC_REG(D31LHOMR),	0,	NULL,			"D31 Low-to-High Output Mask Register"		},
	{ _GSC_REG(D31LHOTSR),	0,	NULL,			"D31 Low-to-High Output Tri-State Register"	},
	{ _GSC_REG(D31LHOVR),	0,	NULL,			"D31 Low-to-High Output Value Register"		},
	{ _GSC_REG(D31HLOMR),	0,	NULL,			"D31 High-to-Low Output Mask Register"		},
	{ _GSC_REG(D31HLOTSR),	0,	NULL,			"D31 High-to-Low Output Tri-State Register"	},
	{ _GSC_REG(D31HLOVR),	0,	NULL,			"D31 High-to-Low Output Value Register"		},

	{ NULL, 0, 0, 0, 0, NULL, NULL	}
};



//*****************************************************************************
static const gsc_reg_def_t* _find_reg(u32 reg, const gsc_reg_def_t* list)
{
	const gsc_reg_def_t*	def	= NULL;
	int						i;

	for (i = 0; list[i].name; i++)
	{
		if (reg == list[i].reg)
		{
			def	= &list[i];
			break;
		}
	}

	return(def);
}



/******************************************************************************
*
*	Function:	dio32_reg_list
*
*	Purpose:
*
*		List the GSC registers and their values.
*
*	Arguments:
*
*		fd		The handle to access the device.
*
*		detail	List the register details?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int dio32_reg_list(int fd, int detail)
{
	int	errs;

	errs	= gsc_reg_list(fd, _gsc, detail, dio32_reg_read);
	return(errs);
}



/******************************************************************************
*
*	Function:	dio32_reg_get_def_id
*
*	Purpose:
*
*		Retrieve the register definition structure given the register id.
*
*	Arguments:
*
*		reg		The id of the register to access.
*
*	Returned:
*
*		NULL	The register id wasn't found.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* dio32_reg_get_def_id(u32 reg)
{
	const gsc_reg_def_t*	def;

	def	= _find_reg(reg, _gsc);
	return(def);
}



/******************************************************************************
*
*	Function:	dio32_reg_get_def_index
*
*	Purpose:
*
*		Retrieve the register definition structure based on an index.
*
*	Arguments:
*
*		index	The index of the register to access.
*
*	Returned:
*
*		NULL	The index doesn't correspond to a known register.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* dio32_reg_get_def_index(int index)
{
	const gsc_reg_def_t*	def;

	if (index < 0)
		def	= NULL;
	else if (index >= (SIZEOF_ARRAY(_gsc) - 1))
		def	= NULL;
	else
		def	= &_gsc[index];

	return(def);
}



/******************************************************************************
*
*	Function:	dio32_reg_get_desc
*
*	Purpose:
*
*		Retrieve the description of the specified register.
*
*	Arguments:
*
*		reg		The register whose description is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* dio32_reg_get_desc(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				desc;

	def	= _find_reg(reg, _gsc);

	if (def)
		desc	= def->desc;
	else
		desc	= "UNKNOWN";

	return(desc);
}



/******************************************************************************
*
*	Function:	dio32_reg_get_name
*
*	Purpose:
*
*		Retrieve the name of the specified register.
*
*	Arguments:
*
*		reg		The register whose name is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* dio32_reg_get_name(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				name;

	def	= _find_reg(reg, _gsc);

	if (def)
		name	= def->name;
	else
		name	= "UNKNOWN";

	return(name);
}


